; ----------------------------------------------------------------------------------------------
;   lcdsr44780.asm                                            Spinelli - 24/10/2008 - 16/06/2015
;
;
;   DRIVER PARA CONTROLADORA COM 2 FIOS PARA LCD HD44780 E SIMILARES.
;
;   Esta controladora usa um CI shift-register do tipo 4015 ou SN74LS164 para receber comandos e
;   dados do PIC e envia-los ao LCD usando penas dois pinos do PIC para fazer a comunicacao.
;
; REQUISITOS DE SOFTWARE
; ----------------------
; INCLUIR NO PROGRAMA: "..\..\SERVICES\delay_services.inc"
;
; AS SEGUINTES VARIAVEIS E SIMBOLOS DEVEM SER DEFINIDAS NO PROGRAMA:
;
;SR_1         RES          1             ; registrador auxiliar 1
;SR_2         RES          1             ; registrador auxiliar 2
;SR_3         RES          1             ; registrador auxiliar 3
;SR_4         RES          1             ; registrador auxiliar 4
;SR_5         RES          1             ; registrador auxiliar 5
;LCD_COMMAND  RES          1             ; buffer de comando e dados
;SR_TIPO      RES          1             ; tipo de LCD
;SR_CLK       equ          7             ; em TRISx este bit deve ser saida
;SR_DAT       equ          6             ; em TRISx este bit deve ser saida
;SR_CHARS     equ          0             ; 0 - caracteres 5x7    1 - caracteres 5x10
;#define      SR_TRIS      TRISA         ; I/O
;#define      SR_DATA      LATA,SR_DAT   ; DATA
;#define      SR_CLOCK     LATA,SR_CLK   ; CLOCK
; ----------------------------------------------------------------------------------------------
;
; EXEMPLO:
;    call      SR_4BITS        ; inicializa LCD 
;    SET_CURSOR .1,.1,"i"      ; posiciona na linha 1, coluna 1 do LCD
;    movlw     "A"             ; caracter "A"
;    call      SR_SENDW        ; move "A" para o LCD
;
; ----------------------------------------------------------------------------------------------
;
;   LOG DAS MODIFICACOES
;   --------------------
;
;   03/07/2016
;   ----------
;   Inclusao das funcoes SR_CURSOR_BLINK e SR_CURSOR_NOBLINK.
;
;   08/07/2015
;   ----------
;   Adicionadas a funcoes 'SR_SCROLL_LEFT' e 'SR_SCROLL_RIGHT' para rolar o display
;   para a esquerda e para a direita respectivamente.
;
;   04/07/2015
;   ----------
;   Adicionada a funcao "lcd_sendstr" e a macro LCD_WRITE para mostrar, no LCD, strings
;   definidas inline na macro. As strings serao definidas na memoria do programa portanto,
;   essa funcao so funciona na familia 18F.
;
;   16/06/2015
;   ----------
;   Este modulo foi copiado para a pasta '\SERVICES\lcd_services.inc' onde, agora, faz parte de
;   um conjunto de outros servicos exaustivamente testados. Em  '\SERVICES\lcd_services.inc'
;   tambem ja estao embutidas as macros 'SR_4MVC', 'LCD_LINES' e 'SET_CURSOR'.
;   Incluir '\SERVICES\lcd_services.inc' no inicio do programa, nao mais no final.
;
;   28/03/2014
;   ----------
;   Inserido o teste [IF __FAMILIA<=16] na funcao SR_4BITS para acessar o registrador SR_TRIS    
;   no banco correto. Milagre!!! Nao sei como essa funcao funcionou ate hoje sem esse teste.
;
;   06/10/2013
;   ----------
;   A macro SR_4MVC passa a usar a funcao SR_SENDW e o registrador SR_3 na familia 16F.
;
;   05/10/2013
;   ----------
;   Inserida a funcao SR_SENDW para enviar ao LCD o caracter contido no registrador W, sem que
;   o programador tenha necessidade de move-lo antecipadamente para o registrador LCD_COMMAND.
;
;   01/10/2013
;   ----------
;   Alteracao da funcao SR_LCDRESET e eliminacao das funcoes SR_SETLINE e SR_SETCOL.
;   Agora os registradores SR_1 e SR_2 ja vem com os valores corrigidos diretamente pela 
;   macro SET_CURSOR.
;   
;   29/04/2013
;   ----------
;   Acrescentado o simbolo SR_TRIS para definir o controle da porta de I/O.
;   A funcao SR_4BITS torna OUTPUT os bits SR_DAT e SR_CLK em SR_TRIS.
;
;   10/11/2012
;   ----------
;   A funcao SR_SETCGRAM passa a fazer chamada a funcao SR_SENDCMD.
;
;   04/11/2012
;   ----------
;   Foi modificada a montagem condicional de __FAMILIA==16 para __FAMILIA<=16, assim 
;   permitindo compilacao para chips de familias 12F.
;   A funcao SR_LCDRESET foi acertada para mover cursor fazendo call para as funcoes SR_SETLINE
;   e SR_SETCOL.
;
;   04/01/2012
;   ----------
;   A chamada para a rotina SR_SCLEAR na rotina SR_MOVE foi eliminada e colocada inline na 
;   na prorpia rotina SR_MOVE porque causava estouro do STACK. Assim, a rotina SR_SCLEAR
;   deixou de ser necessaria e foi eliminada deste modulo.
;
;   12/09/2011
;   ----------
;   Modificacao das rotinas STROBE e DSTROBE que agora passam a ser temporizadas com o valor
;   especificado no simbolo "SR_STROBETIME", qualquer que seja a frequencia de clock utilizada
;   nos projetos.
;
;   27/08/2011
;   ----------
;   A funcao SR_LCDCLEAR, SR_CURSORHOME e SR_SETCURSOR passaram a fazer chamada a funcao SR_SENDCMD.
;
;   06/08/2011
;   ----------
;   Incluido teste de de tipo de microcontrolador para gerar codigo adequado para familias
;   diferente.
;
;   02/08/2011
;   ----------
;   Modificacao para as rotinas STROBE e DSTROBE suportarem clock maior do que 20MHZ.
;
;   04/06/2011
;   ----------
;   Incluida a funcao SR_SETCGRAM para estabelecer o endereco da CGRAM quando se deseja definir
;   caracteres de usuarios.
;   Incluido o simbolo SR_CHARS para permitir selecionar entre LCDs com caracteres 5x7 e 5x10
;   pixels durante a inicializacao do LCD. SR_CHARS sera opcional. Quando definido no programa
;   com valor zero, oo quando omitido, sera selecionado o modo 5x7. Quando definido com valor 1,
;   sera selecionado modo 5x10. A alteracao foi realizada na funcao de SR_4BITS com montagem 
;   condicional na chamada da funcao SR_FUNCTION_5X7 ou SR_FUNCTION_5X10.
;
;   25/05/2011
;   ----------
;   A partir desta modificacao, estas subrotinas passam a suportar LCDs de 2x16, 4x16 e 4x20.
;   Para corrigir a falha de nao conseguir enderecar mais do que 16 colunas, foram inseridas as 
;   subrotinas "SR_setline" e "SR_setcol".
;   A funcao SR_setcursor de 20/03/2009 foi substituida pela 25/05/2011 para suportar as
;   familias de chip 16F e 18F, bem como para corrigir falhas na velha macro LCD_ADDRESS
;   que foi substituida pela LCD_LINES. 
;
;   17/05/2011
;   As funcoes SR_setcursor e SR_lineset foram modificadas para suportar a leitura da tabela
;   LCD_ADDRESS atraves da insrucao TBLRD. A macro LCD_ADDRESS.mac tambem foi alterada.
;
;   16/05/2011
;   ----------
;   Inserida montagem condicional para ver se esta compilando chip PIC18F4520 nas rotinas SR_MOVE e
;   SR_SEND. O teste monta as instrucoes RRCF e RLCF ao inves de RRF e RLF.
;   Todos os desvios do tipo "goto $+x" foram substituidos por "goto label". Desvios "goto $" nao
;   sao suportados pelos chips PICF18xxx.
;
;   01/08/2010
;   ----------
;   Alteracao das rotinas de tempo.
;   A partir desta versao, serao utilizadas as rotinas de temporizacao externas "delay_ms.inc"
;   e "delay_us.inc".
;   A rotina SR_COPY foi retirada, para economia de memoria, porque nao tinha utilidade.
;   Os registradores R0, R1, R8, R12 e R13 foram substituidos por SR_1 a SR_5 que agora devem ser definidos
;   pelo programador e nao mais pela macro "DEFINES.inc".
;   O registrador "LCD_COMMAND" passa a ser definido pelo programador, nao mais pela macro "DEFINES.inc".
;
;   26/07/2009
;   ----------
;   Esta versao foi alterada para poder especificar DATA e CLOCK em qualquer porta. 
;   Antes so era possivel especifica-los na mesma porta. Ver requisitos de software.
;
;   24/07/2009
;   ----------
;   A macro LCD_ADDRESS foi alterada para poder especificar tipos de LCD (linhas X colunas) e tambem
;   passou a gerar enderecos para LCDs de 4X20.
;
;   24/04/2009
;   ----------
;   Modificacao para funcionar com cristais de 3,57581MHz, 3,579545MHz, 4MHz, 7MHz, 16,9344MHz e 20MHz.
;   Ao usar cristais externos deve-se especificar  __CONFIG _HS_OSC para especificar cristais de 
;   alta velocidade.
;
;   20/02/2009
;   ----------
;   Introducao da macro LCD_ADDRESS para gerar tabela de enderecos de linhas do LCD usados
;   pela subrotina SR_lineset para posicionar o cursor no inicio de uma determinada linha.
;   A sub-rotina SR_setcursor passou a aceitar LCD de 4 linhas.
;
;   16/12/2008
;   ----------
;   Introducao da sub-rotina SR_COPY para usar buffer de LCD na memoria.
;
;
; ----------------------------------------------------------------------------------------------
 

; ----------------------------------------------------------------------------------------------
;
;          ROTINAS PARA USO DO PROGRAMADOR
;          -------------------------------
;          SR_4BITS........................ (U) inicializa o LCD
;          SR_SENDCMD...................... (U) envia o comando  carregado em LCD_COMMAND
;          SR_SENDCHR...................... (U) envia o caracter carregado em LCD_COMMAND
;          SR_LCDCLEAR..................... (U) apaga o conteudo da tela do LCD
;          SR_lineset...................... (U) posiciona o cursor no inicio de uma linha
;          SR_cursorhome................... (U) monta, variavel LCD_COMMAND, o comando para colocar
;                                           o cursor na linha 1 coluna 1. Deve ser seguido de
;                                           uma chamada a rotina SR_SENDCMD.
;          SR_LCDRESET..................... Limpa o LCD e posiciona o cursor na linha 1 e coluna 0  
;          SR_setcursor.................... (U) monta, variavel LCD_COMMAND, o comando para colocar
;                                           o cursor na linha e coluna desejadas.
;                                           Deve ser seguido de uma chamada a rotina SR_SENDCMD.
;          SR_CURSOR_BLINK ................ Cursor piscante
;          SR_CURSOR_NOBLINK .............. Cursor nao piscante

;
;          ROTINAS DE USO INTERNO
;          ----------------------
;          SR_STROBE....................... (I) aciona o strobe do shift register "clockA" para 
;                                           permitir a entrada do bit em "dataA"
;          SR_DSTROBE...................... (I) simula o sinal de strobe (dataA 0->1->0) para o LCD
;                                           executar a instrucao que esta no shift register
;          SR_wait......................... (I) aguarda 1mS
;          SR_wait160...................... (I) aguarda 160uS
;          SR_SENDG........................ (I) rotina interna 
;          SR_SEND......................... (I) rotina interna
;          SR_MOVE......................... (I) move 8 bits para o shift register
;          SR_FUNCTION_CMD................. (I) rotina de inicializacao estabelece valor em LCD_COMMAND
;          SR_FUNCTION_8B.................. (I) rotina de inicializacao estabelece valor em LCD_COMMAND
;          SR_FUNCTION_4B.................. (I) rotina de inicializacao estabelece valor em LCD_COMMAND
;          SR_FUNCTION_2L.................. (I) rotina de inicializacao estabelece valor em LCD_COMMAND
;          SR_FUNCTION_1L.................. (I) rotina de inicializacao estabelece valor em LCD_COMMAND
;          SR_FUNCTION_5X10................ (I) rotina de inicializacao estabelece valor em LCD_COMMAND
;          SR_FUNCTION_5X7................. (I) rotina de inicializacao estabelece valor em LCD_COMMAND
;          SR_DISPLAY_CMD.................. (I) rotina de inicializacao estabelece valor em LCD_COMMAND
;          SR_DISPLAY_ON................... (I) rotina de inicializacao estabelece valor em LCD_COMMAND
;          SR_DISPLAY_OFF.................. (I) rotina de inicializacao estabelece valor em LCD_COMMAND
;          SR_DISPLAY_CURSOR_ON............ (I) rotina de inicializacao estabelece valor em LCD_COMMAND
;          SR_DISPLAY_CURSOR_OFF........... (I) rotina de inicializacao estabelece valor em LCD_COMMAND
;          SR_DISPLAY_BLINK................ (I) rotina de inicializacao estabelece valor em LCD_COMMAND
;          SR_DISPLAY_NOBLINK.............. (I) rotina de inicializacao estabelece valor em LCD_COMMAND
;          SR_ENTRYMODE_CMD................ (I) rotina de inicializacao estabelece valor em LCD_COMMAND
;          SR_ENTRYMODE_SHIFT.............. (I) rotina de inicializacao estabelece valor em LCD_COMMAND
;          SR_ENTRYMODE_NOSHIFT............ (I) rotina de inicializacao estabelece valor em LCD_COMMAND
;          SR_ENTRYMODE_ID................. (I) rotina de inicializacao estabelece valor em LCD_COMMAND
;          SR_ENTRYMODE_NOID............... (I) rotina de inicializacao estabelece valor em LCD_COMMAND
;          SR_SHIFT_CMD.................... (I) rotina de inicializacao estabelece valor em LCD_COMMAND
;          SR_SHIFT........................ (I) rotina de inicializacao estabelece valor em LCD_COMMAND
;          SR_SHIFT_CMOVE.................. (I) rotina de inicializacao estabelece valor em LCD_COMMAND
;          SR_SHIFT_RIGHT.................. (I) rotina de inicializacao estabelece valor em LCD_COMMAND
;          SR_SHIFT_LEFT .................. (I) rotina de inicializacao estabelece valor em LCD_COMMAND
;
; ----------------------------------------------------------------------------------------------
 
 
; ----------------------------------------------------------------------------------------------
;                         HARDWARE NECESSARIO
; ----------------------------------------------------------------------------------------------
;                        +---------------+               +---------------+  
;                        |               |               |               | 
;                        |             12[]b2-------->D7[]14             | 
;                        |               |               |               | 
;             (+5V)  VDD[]16           15[]Db---+        |             15[] (+) BACK LIGHT
;                        |               |      |        |               | 
;              (-)   VSS[]8            13[]b1-------->D6[]13           16[} (-) BACK LIGHT         o +5V
;                        |     SHIFT     |      |        |               |                         | 
;              +----CLKB[]1            10[]a4---+---->D5[]12            1[]  (-) CONTRAST          |
; +-----+      |         |    REGISTER   |               |     L C D     |                       +-+-+ 
; |     +--> >-+---CLKA[]9              3|[]a3------->D4[]11            2[]  (+) VDD             |   | 
; | PIC |                |     4015      |               |               |                       |   |
; |     +--> >-+-------Da[]7            11[]b3------->RS[]4             3[]  Vo ---------------->|   | 1K
; +-----+      |         |               |               |               |                       |   |
;              |  +-RSTA[]6             2[]b4-+   +-->EN[]6              |                       |   |
;              |  |      |               |    |   |      |               |                       +-+-+ 
;              |  +-RSTB[]14             |    |   | +-RW[]5              |                         | 
;              |  |      |               |    _   | |    |               |                         | 
;              | ///     +---------------+  D1^   | |    +---------------+                         |
;              |                              |   | |                                             ///
;              |   +---------+                |   | | 
;              +---+   1K    +----------------+---+ |          +-------------------------------------------+
;                  +---------+                      |          | Nota: R1 e D1 formam uma porta AND        |
;                      R1                          ///         +-------------------------------------------+
;   R1=1K para LCD HD44780                                     | Se DATAA=0  e B4=0 ===> EN=0              |
;   R1>4K para LCD 1602ED                                      | Se DATAA=1  e B4=0 ===> EN=0              |
;                                                              | Se DATAA=0  e B4=1 ===> EN=0              |
;                                                              | Se DATAA=1  e B4=1 ===> EN=1 --> STROBE   |
;                                                              +-------------------------------------------+
;
;
;              B4   B3  B2  B1  A4  A3  A2  A1
;             +---+----+---+---+---+---+---+---+                         +-------------------------------+
; SHIFT REG   | 1 | RS | X | X | X | X | 0 | 0 | ---> COMANDO PARA O LCD |  B4 -> SEMPRE 1               |
;             +---+----+---+---+---+---+-+-+---+                         |  B3 -> RS=0 CMD ou RS=1 DATA  |
;               |    |   |   |   |   |   |                               |  <B2:A1> -> valor             |
;               |    |   |   |   |   |   |                               |  A2 -> NAO USADO              |
;               |    |   |   |   |   |   |                               |  A1 -> NAO USADO              |
;               |    |   |   |   |   |   |                               +-------------------------------+
;               |    |   |   |   |   |   |                                
;               |    |   |   |   |   |   +----------> 1 - Fazer o pino DATAA=0 antes deste bit 
;               |    |   |   |   |   |                    entrar no shift register.
;               |    |   |   |   |   |                2 - Dar 2 strobes seguidos em CLKA.
;             +---+----+---+---+---+---+              3 - Quando o shift register estiver completo,
;         LCD |EN | RS | D7| D6| D5| D4|                  fazer o pino DATAA=1 momentaneamente para simular
;             +---+----+---+---+---+---+                  um strobe para o LCD.
;               6    4   14  13  12  11
; --------------------------------------------------------------------------------------

; --------------------------------------------------------------------------------------
; ROLAGEM DO DISPLAY PARA A ESQUERDA                                          08/07/2015
; --------------------------------------------------------------------------------------
SR_SCROLL_LEFT
           call     SR_SHIFT_CMD
           call     SR_SHIFT 
           call     SR_SHIFT_LEFT
           call     SR_SENDCMD
           return
; --------------------------------------------------------------------------------------
; ROLAGEM DO DISPLAY PARA A DIREITA                                           08/07/2015
; --------------------------------------------------------------------------------------
SR_SCROLL_RIGHT
           call     SR_SHIFT_CMD
           call     SR_SHIFT 
           call     SR_SHIFT_RIGHT
           call     SR_SENDCMD
           return
; --------------------------------------------------------------------------------------
;          INICIALIZACAO DO LCD                                              27/10/2008
;
;          Esta rotina monta e envia os comandos de inicializacao do LCD.
;          Cada comando tem o formato b'ERxxxx00' onde: 
;          E ........ EN do LCD 
;          R ........ RS do LCD (aqui sera sempre zero)
;          xxxx ..... comando
;
;          Entrada: nao ha
;          Saida  : LCD inicializado
;
;          TEMPO: 4,78mS a 4MHz
; --------------------------------------------------------------------------------------
SR_STROBETIME equ   .50                   ; tempo do strobe em micro segundos
            ; -------------------------------------------------------------------------- 
            ; Primeiro envia a sequencia de reset do LCD:  0x03, 0x03, 0x03, 0x02 
            ; e depois os comandos de inicializacao
            ; -------------------------------------------------------------------------- 
SR_4BITS    
   IF __FAMILIA<=16
            bsf      STATUS,RP0
   ENDIF   
            bcf      SR_TRIS,SR_DAT       ; SR_DAT --> saida
            bcf      SR_TRIS,SR_CLK       ; SR_CLK --> saida
   IF __FAMILIA<=16
            bcf      STATUS,RP0
   ENDIF   
            movlw    b'10001100'          ; EN=1, RS=0, xxxx=3 
            movwf    SR_4                 ; 
            call     SR_MOVE              ; vai enviar ao shift register
            call     SR_DSTROBE           ; vai simular strobe para o LCD
            movlw    .5                   ; . 
            movwf    SR_2                 ;  .
SR_4B_2     call     SR_wait              ;   . aguarda 5mS
            decfsz   SR_2,f               ;  .
            goto     SR_4B_2              ; .
;
            movlw    b'10001100'          ; EN=1, RS=0, xxxx=3 
            movwf    SR_4                 ;
            call     SR_MOVE              ; vai enviar ao shift register
            call     SR_DSTROBE           ; vai simular strobe para o LCD
            call     SR_wait160           ; aguarda 160uS
;
            movlw    b'10001100'          ; EN=1, RS=0, xxxx=3 
            movwf    SR_4                 ;
            call     SR_MOVE              ; vai enviar ao shift register
            call     SR_DSTROBE           ; vai simular strobe para o LCD
            call     SR_wait160           ; aguarda 160uS
;
            movlw    b'10001000'          ; EN=1, RS=0, xxxx=2 
            movwf    SR_4                 ;
            call     SR_MOVE              ; vai enviar ao shift register
            call     SR_DSTROBE           ; vai simular strobe para o LCD
            call     SR_wait160           ; aguarda 160uS
            ; -------------------------------------------------------------------------- 
            ; inicializa o LCD
            ; -------------------------------------------------------------------------- 
            call     SR_FUNCTION_CMD        ; registra function
            call     SR_FUNCTION_4B         ; interface de 4 bits
            call     SR_FUNCTION_2L         ; LCD de duas linhas

  IFDEF SR_CHARS
    LOCAL   SR_c  = SR_CHARS
    IF SR_c == 1
            call     SR_FUNCTION_5X10        ; matriz de 5X10
    ELSE
            call     SR_FUNCTION_5X7         ; matriz de 5X7
    ENDIF
  ELSE
            call     SR_FUNCTION_5X7         ; matriz de 5X7
  ENDIF


            call     SR_SENDCMD             ; vai enviar comando 
;
            call     SR_DISPLAY_CMD         ; registra display
            call     SR_DISPLAY_OFF         ; desliga o LCD
            call     SR_DISPLAY_CURSOR_OFF  ; desliga o cursosr
            call     SR_DISPLAY_NOBLINK     ; nao pisca
            call     SR_SENDCMD             ; vai enviar comando
;
            call     SR_DISPLAY_CMD         ; registra display
            call     SR_DISPLAY_ON          ; liga o LCD
            call     SR_DISPLAY_CURSOR_ON   ; liga o cursor
            call     SR_DISPLAY_NOBLINK       ; cursor piscante
            call     SR_SENDCMD             ; vai enviar comando 
;
            call     SR_ENTRYMODE_CMD       ; registra entry mode
            call     SR_ENTRYMODE_NOSHIFT   ; nao deslocar
            call     SR_ENTRYMODE_ID        ; incrementar posicao do cursor
            call     SR_SENDCMD             ; vai enviar comando
;
            movlw    .250
            call     delay_ms
            return                          ; fim
 


; --------------------------------------------------------------------------------------
;                                                                            28/10/2008
;          ENVIA UM COMANDO OU CARACTER DE 8 BITS QUE ESTA NA VARIAVEL LCD_COMMAND
;          AO LCD USANDO A INTERFACE DE 4 BITS. O COMANDO SERA DIVIDIDO EM DOIS NIBLES.
;          O NIBLE MSB SERA ENVIADO PRIMEIRO SEGUIDO DO NIBLE LSB.
;
;          T=623uS a 4MHz
; --------------------------------------------------------------------------------------
            ; --------------------------------------------------------------------------
            ; ESTABELECE MODO COMANDO  ---> (RS=0)                           28/10/2008
            ; --------------------------------------------------------------------------
SR_SENDCMD  movlw    b'10000000'          ;   W = 1000.0000    ESR_10.0000
            movwf    SR_TIPO              ; TIPO= 1000.0000    ESR_10.0000
            goto     SR_SENDG
            ; --------------------------------------------------------------------------
            ; ESTABELECE MODO CARACTER ---> (RS=1)                           28/10/2008
            ; --------------------------------------------------------------------------
SR_SENDW    movwf    LCD_COMMAND          ; Funcao SR_SENDW
                                          ; esta funcao foi inserida em 05/10/2013
SR_PUT
SR_SENDCHR  movlw    b'11000000'          ;   W = 1100.0000    ESR_10.0000
            movwf    SR_TIPO              ; TIPO= 1100.0000    ESR_10.0000
            ; --------------------------------------------------------------------------
            ;           PREPARA E ENVIA O MSB                                28/10/2008 
            ; --------------------------------------------------------------------------
SR_SENDG    movf     LCD_COMMAND,w        ;   W = MMMM.LLLL
            movwf    SR_2                 ;  SR_2 = MMMM.LLLL  <----- MSB 
            call     SR_SEND              ; envia o nible MSB      
            ; --------------------------------------------------------------------------
            ;           PREPARA E ENVIA O LSB                                28/10/2008
            ; --------------------------------------------------------------------------
            movf     LCD_COMMAND,w        ;   W = MMMM.LLLL
            movwf    SR_2                 ;  SR_2 = MMMM.LLLL
            swapf    SR_2,f               ;  SR_2 = LLLL.MMMM  <----- LSB 
            call     SR_SEND              ; envia o nible LSB 
            call     SR_wait              ; aguarda  --------------------------------------> SIM
            return                        ; fim
; --------------------------------------------------------------------------------------        
;           ENVIA O NIBLE MSB DE SR_2 AO LCD                                   28/10/2008
;           SR_2 = AAAA.xxxx ao entrar nesta rotina onde AAAA sera enviado
;           T=304uS a 4MHz
; --------------------------------------------------------------------------------------
SR_SEND     movf     SR_TIPO,w            ;   W = ESR_10.0000 
            movwf    SR_4                 ; SR_4 = ESR_10.0000    C=?

   IF __FAMILIA==18
            rrcf      SR_2,f              ;  SR_2 = ?AAA.Axxx    C=x 
            rrcf      SR_2,f              ;  SR_2 = x?AA.AAxx    C=x 
   ENDIF

   IF __FAMILIA<=16
            rrf      SR_2,f               ;  SR_2 = ?AAA.Axxx    C=x 
            rrf      SR_2,f               ;  SR_2 = x?AA.AAxx    C=x 
   ENDIF
            movlw    0x3C                 ;   W = 0011.1100
            andwf    SR_2,w               ;   W = 00AA.AA00
            iorwf    SR_4,f               ; SR_4 = 10AA.AA00
            call     SR_MOVE              ; SR_4 = ERAA.AA00
            call     SR_DSTROBE           ; vai simular strobe para o LCD
            return
;
; --------------------------------------------------------------------------------------
;          MOVE 8 BITS DE SR_4 PARA O SHIFT REGISTER                          27/10/2008
;          INICIANDO PELO BIT DE MAIS ALTA ORDEM (MSB)
;          Entrada: SR_4 contem o byte
;          Saida  : Shift register contem o valor passado em SR_4 (SR_4 = abcd.efgh)
;          T=123uS a 4MHz
; --------------------------------------------------------------------------------------
SR_MOVE    movlw   .8                    ; 8 bits
           movwf   SR_2                  ; carrega o contador
           bcf     SR_DATA               ; dado= 0
SR_MOVE2x  call    SR_STROBE             ; enfia zero no shift-register
           decfsz  SR_2,f                ; conta bits
           goto    SR_MOVE2x             ; proximo bit
;
           movlw    .8                   ; .8 bits
           movwf    SR_5                 ; mover 8 bits                    
SR_MOVE1   bcf      SR_DATA              ; SR_DATA = 0

   IF __FAMILIA==18
            rlcf      SR_4,f               ;  
   ENDIF

   IF __FAMILIA<=16
            rlf      SR_4,f               ;  
   ENDIF
            btfsc    STATUS,C   
            bsf      SR_DATA              ; SR_DATA = 1
            call     SR_STROBE            ; strobe para o shift register
            decfsz   SR_5,f               ; 
            goto     SR_MOVE1
            return
;
; --------------------------------------------------------------------------------------
;          LEVANTA E ABAIXA O SINAL DE STROBE (clockA) DO SHIFT REGISTER     27/10/2008
;          PARA MOVER UM BIT DO PIC PARA O SHIGT REGISTER.
; --------------------------------------------------------------------------------------
SR_STROBE    bsf     SR_CLOCK
             movlw   SR_STROBETIME
             call    delay_us
             bcf     SR_CLOCK
             return
;
; ---------------------------------------------------------------------------------------------------------
;          SIMULA O SINAL DE STROBE (DATAA) PARA O LCD                                          28/10/2008
;          EXECUTAR A INSTRUCAO CONTIDA NO SHIFT REGISTER.
; ---------------------------------------------------------------------------------------------------------
SR_DSTROBE   bsf      SR_DATA      ; levanta o DATAA 
             movlw   SR_STROBETIME
             call    delay_us
             bcf     SR_DATA
             return

; ---------------------------------------------------------------------------------------------------------
; SR_wait - Aguarda 1mS (mili segundo) para o LCD terminar o comando                            24/10/2008
; Alterado na versao 2 em 01/08/2010
; ---------------------------------------------------------------------------------------------------------
SR_wait      movlw   .1
             call    delay_ms
             return
;
; ---------------------------------------------------------------------------------------------------------
; SR_wait160 - Aguarda 160uS (micro segundos) para o LCD terminar o comando                     27/10/2008
; Alterado na versao 2 em 01/08/2010
; ---------------------------------------------------------------------------------------------------------
SR_wait160   movlw   .160
             call    delay_us
             return
;
; ---------------------------------------------------------------------------------------------------------
; SR_cursorhome - Envia o cursor para o inicio                                                 28/10/2008
; T=4uS a 4MHz
; ---------------------------------------------------------------------------------------------------------
SR_CURSORHOME
SR_cursorhome  movlw   b'00000010'       ; comando "cursor home"
               movwf   LCD_COMMAND       ; armazena o comando
               call    SR_SENDCMD
               return                    ; fim
;
; ---------------------------------------------------------------------------------------------------------
; SR_LCDRESET- Limpa o LCD e posiciona o cursor na linha 1 e coluna 0                          10/12/2008
; ---------------------------------------------------------------------------------------------------------
SR_LCDRESET    call   SR_LCDCLEAR
               clrf   SR_1                        ; incluido 01/10/2013
               clrf   SR_2                        ; incluido 01/10/2013
               call   SR_setcursor
               return
; ---------------------------------------------------------------------------------------------------------
; SR_LCDCLEAR- Limpa o LCD                                                                       28/10/2008
; T=4uS a 4MHz
; ---------------------------------------------------------------------------------------------------------
SR_CLEAR
SR_LCDCLEAR  movlw   0x01                ; comando 01 - CLEAR
             movwf   LCD_COMMAND         ; armazena o comando
             call    SR_SENDCMD
             return                      ; fim


; ---------------------------------------------------------------------------------------------------------
; SR_lineset                                                                         Spinelli - 20/02/2009
; Posiciona o cursor na primeira coluna de uma determinada linha.
; W deve conter o numero da linha.
; 
; Por enquanto suporta LCDs de ate 4 linhas.
;
; Esta rotina exige que a macro LCD_ADDRESS seja emitida no inicio do programa para gerar uma tabela de
; enderecos das linhas do LCD.
;
; Exemplo:
;            movlw   .2
;            call    SR_lineset
;
; ---------------------------------------------------------------------------------------------------------
     IF __FAMILIA==18
SR_LINESET
SR_lineset   movwf   SR_1
             movlw   UPPER(LCD_ADDRTAB)
             movwf   TBLPTRU
             movlw   HIGH(LCD_ADDRTAB)
             movwf   TBLPTRH
             movlw   LOW(LCD_ADDRTAB)
             addwf   SR_1,w
             movwf   TBLPTRL
             tblrd   *+
             movf    TABLAT,w
             movwf   LCD_COMMAND       ; move SET DDRAM p/ buffer de comando
             return
  ENDIF

   IF __FAMILIA<=16
SR_LINESET
SR_lineset   call    LCD_ADDRTAB       ; busca o endereco da linha
             movwf   LCD_COMMAND       ; move SET DDRAM p/ buffer de comando
             return                    ; retorna
  ENDIF



; ---------------------------------------------------------------------------------------------------------
; SR_setcursor - Posiciona o cursor em LCDS 2x16, 4x16 e 4x20                         Spinelli - 25/03/2011
;
; Esta rotina exige que a macro LCD_LINES seja emitida no inicio do programa para gerar a tabela de
; enderecos das linhas do LCD como a seguir:
;
; ENTRADA: SR_1 contem o numero da linha
;          SR_2 contem o numero da coluna
; 
; SAIDA  : LCD posicionado
;
;
; ---------------------------------------------------------------------------------------------------------
SR_SETCURSOR
SR_setcursor   

   IF __FAMILIA==18
             movlw   UPPER(LCD_ADDRTAB)          ; endereco da tabela
             movwf   TBLPTRU                     ; carrega apontador
             movlw   HIGH(LCD_ADDRTAB)           ; endereco da tabela
             movwf   TBLPTRH                     ; carrega apontador
             movlw   LOW(LCD_ADDRTAB)            ; endereco da tabela
             addwf   SR_1,w                      ; calcula o deslocamento na tabela
             movwf   TBLPTRL                     ; estabelece o endereco
             tblrd   *+                          ; le o endereco da linha
             movf    TABLAT,w                    ; pega o endereco lido
             addwf   SR_2,w                      ; adiciona o numero da coluna
             iorlw   0x80                        ; mistura endereco calculado com o comando
             movwf   LCD_COMMAND                 ; insere comando no buffer
             call    SR_SENDCMD
             return
  ENDIF

   IF __FAMILIA<=16
             movf    SR_1,w                      ; pega o numero da linha
             call    LCD_ADDRTAB                 ; le o endereco na tabela
             addwf   SR_2,w                      ; adiciona o numero da coluna
             iorlw   0x80                        ; mistura endereco calculado com o comando
             movwf   LCD_COMMAND                 ; insere comando no buffer
             call    SR_SENDCMD
             return
  ENDIF


;

; ---------------------------------------------------------------------------------------------------------
;                COMANDO SET CGRAM                                                              04/06/2011
; ENTRADA: W contem o endereco da CGRAM a ser definido no LCD
; ---------------------------------------------------------------------------------------------------------
SR_SETCGRAM      iorlw    0x40                   ; mistura endereco com o comando
                 movwf    LCD_COMMAND
                 call     SR_SENDCMD
                 movlw    .30
                 call     delay_ms
                 return                          ;




; ---------------------------------------------------------------------------------------------------------
;                COMANDO SET FUNCTION                                                           24/10/2008
; ---------------------------------------------------------------------------------------------------------
SR_FUNCTION_CMD  movlw    b'00100000'         ; FUNCTIOM CMD
                 movwf    LCD_COMMAND
                 return
SR_FUNCTION_8B   movlw    b'00110000'         ; interface de 8 bits
                 iorwf    LCD_COMMAND,f
                 return
SR_FUNCTION_4B   movlw    b'00101111'         ; interface de 4 bits
                 andwf    LCD_COMMAND,f
                 return
SR_FUNCTION_2L   movlw    b'00101000'         ; display de 2 linhas
                 iorwf    LCD_COMMAND,f
                 return
SR_FUNCTION_1L   movlw    b'00110111'         ; display de 1 linha
                 andwf    LCD_COMMAND,f
                 return
SR_FUNCTION_5X10 movlw  b'00100100'           ; matriz 5X10
                 iorwf    LCD_COMMAND,f
                 return
SR_FUNCTION_5X7  movlw  b'00111011'           ; matriz 5X7
                 andwf    LCD_COMMAND,f
                 return
; ---------------------------------------------------------------------------------------------------------
;                COMANDO DISPLAY                                                                24/10/2008
; ---------------------------------------------------------------------------------------------------------
SR_DISPLAY_CMD   movlw    b'00001000'     ; DISPLAY CMD
                 movwf    LCD_COMMAND
                 return
SR_DISPLAY_ON    movlw    b'00001100'     ; DISPLAY ON
                 iorwf    LCD_COMMAND,f
                 return
SR_DISPLAY_OFF   movlw    b'00001011'     ; DISPLAY OFF
                 andwf    LCD_COMMAND,f
                 return
SR_DISPLAY_CURSOR_ON movlw b'00001010'   ; CURSOR ON
                 iorwf    LCD_COMMAND,f
                 return
SR_DISPLAY_CURSOR_OFF movlw b'00001101'  ; CURSOR OFF
                 andwf    LCD_COMMAND,f
                 return
SR_DISPLAY_BLINK movlw   b'00001001'     ; BLINK
                 iorwf    LCD_COMMAND,f
                 return
SR_DISPLAY_NOBLINK movlw   b'00001110'   ; NO BLINK
                 andwf    LCD_COMMAND,f
                 return
; ---------------------------------------------------------------------------------------------------------
;                COMANDO ENTRY MODE                                                             24/10/2008
; ---------------------------------------------------------------------------------------------------------
SR_ENTRYMODE_CMD movlw     b'00000100'        ; ENTRY MODE CMD
                 movwf      LCD_COMMAND
                 return
SR_ENTRYMODE_SHIFT movlw   b'00000101'        ; display shifted
                 iorwf      LCD_COMMAND,f
                 return
SR_ENTRYMODE_NOSHIFT movlw b'00000110'        ; display noshifted
                 andwf      LCD_COMMAND,f
                 return
SR_ENTRYMODE_ID  movlw      b'00000110'        ; increase
                 iorwf      LCD_COMMAND,f
                 return
SR_ENTRYMODE_NOID movlw    b'00000101'         ; decrease
                 andwf      LCD_COMMAND,f
                 return
; ---------------------------------------------------------------------------------------------------------
;                COMANDO SHIFT                                                                  24/10/2008
; ---------------------------------------------------------------------------------------------------------
SR_SHIFT_CMD     movlw     b'00010000'         ; SHIFT COMMAND
                 movwf     LCD_COMMAND
                 return
SR_SHIFT         movlw     b'00011000'         ; Display shift
                 iorwf     LCD_COMMAND,f
                 return
SR_SHIFT_CMOVE   movlw     b'00010111'         ; Display shift cursor move
                 andwf     LCD_COMMAND,f
                 return
SR_SHIFT_RIGHT   movlw     b'00010100'
                 iorwf     LCD_COMMAND,f       ; Display shift right
                 return
SR_SHIFT_LEFT    movlw     b'00011011'         ; Display shift left
                 andwf     LCD_COMMAND,f
                 return


;
; ---------------------------------------------------------------------------------------------------------
; CURSOR PISCANTE                                                                                03/07/2016
; ---------------------------------------------------------------------------------------------------------
SR_CURSOR_BLINK
            call     SR_DISPLAY_CMD         ; registra display
            call     SR_DISPLAY_ON          ; liga o LCD
            call     SR_DISPLAY_CURSOR_ON   ; liga o cursor
            call     SR_DISPLAY_BLINK       ; cursor piscante
            call     SR_SENDCMD             ; vai enviar comando 
            return
; ---------------------------------------------------------------------------------------------------------
; CURSOR NAO PISCANTE                                                                            03/07/2016
; ---------------------------------------------------------------------------------------------------------
SR_CURSOR_NOBLINK
            call     SR_DISPLAY_CMD         ; registra display
            call     SR_DISPLAY_ON          ; liga o LCD
            call     SR_DISPLAY_CURSOR_ON   ; liga o cursor
            call     SR_DISPLAY_NOBLINK       ; cursor piscante
            call     SR_SENDCMD             ; vai enviar comando 
            return
;
; -------------------------------------------------------------------------------------------------------------------------
;              LCD - FORMATO DOS COMANDOS
; -------------------------------------------------------------------------------------------------------------------------
;              RS  RW  DB7  DB6  DB5  DB4  DB3  DB2  DB1  DB0 
; -------------------------------------------------------------------------------------------------------------------------
; DISPLAY      L   L    L    L    L    L    L    L    L    H         
; CLEAR
; -------------------------------------------------------------------------------------------------------------------------
; RETURN       L   L    L    L    L    L    L    L    H    X         move cursor para o digito 1
; HOME
; -------------------------------------------------------------------------------------------------------------------------
; ENTRY        L   L    L    L    L    L    L    H    ID   SH        ID -> H=increase              L=decrease
; MODE                                                               SH -> H=display shifted       L=display not shifted
; -------------------------------------------------------------------------------------------------------------------------
; DISPLAY      L   L    L    L    L    L    H    D    C    B          D -> H=display on            L=display off
; ON/OFF                                                              C -> H=cursor on             L=cursor off
;                                                                     B -> H=blinking on           L=blinking off
; -------------------------------------------------------------------------------------------------------------------------
; SHIFT        0   0    0    0    0    1    SC   RL   X    X         SC -> 1=display shift         0=cursor move
;                                                                    RL -> 1=right shift           0=left shift
; -------------------------------------------------------------------------------------------------------------------------
; SET          L   L    L    L    H    DL   N    F    X    X         DL -> H=8 bits interface      L=4 bits interface
; FUNCTION                                                            N -> H=display de 2 linhas   L=dsiplay de 1 linha
;                                                                     F -> H=matriz 5X10           L=matriz 5X7
; -------------------------------------------------------------------------------------------------------------------------
; SET CG RAM   L   L    L    H    |<--- CG RAM ADDRESS ---->|        CG RAM data enviado e recebido apos este ordem
; ADDRESS
; -------------------------------------------------------------------------------------------------------------------------
; SET DD RAM   L   L    H    |<----- DD RAM ADDRESS ------->|        DD RAM data enviado e recebido apos este ordem
; ADDRESS
; -------------------------------------------------------------------------------------------------------------------------
; READ BUSY    L   H    BF   |<-- ADDRESS COUNTER USED ---->|        BF -> H=busy                  L=ready
; FLAG &                     | FOR BOTH DD & CG RAM ADDRESS |              BF indica a leitura do status do LCD
; ADDRESS
; -------------------------------------------------------------------------------------------------------------------------
; WRITE DATA   H   L    |<------------ data --------------->|        grava dado em DD ou CG RAM
; -------------------------------------------------------------------------------------------------------------------------
; READ  DATA   H   L    |<------------ data --------------->|        le daoa em DD ou CG RAM
; -------------------------------------------------------------------------------------------------------------------------
;







 IF __FAMILIA==18
; ---------------------------------------------------------------------------------------------
; lcd_sendstr - imprime, no terminal, uma string terminada por nulo (0x00).          04/07/2015
;
; Este codigo sera gerado apenas para a familia 18F.
;
; A string deve ser definida por 'DW' logo apos a chamada da funcao 'lcd_sendstr'.
;
; Exemplo:     call    lcd_sendstr
;              DW      "Ola!!!",0
;
; A macro LCD_WRITE pode ser usada para fazer a chamada a esta funcao.
;
; ---------------------------------------------------------------------------------------------
lcd_sendstr    movff   TOSU,TBLPTRU              ; topo do stack para UPPER
               movff   TOSH,TBLPTRH              ; topo do stack para HIGH
               movff   TOSL,TBLPTRL              ; topo do stack para LOW
               call   _lcdsend_str_c             ; vai imprimir
               btfsc   TBLPTRL,0                 ; endereco par???
               tblrd   *+                        ; NAO --> avanca para endereco par
               movf    TBLPTRU,w                 ; restaura o topo do stack
               movwf   TOSU                      ; restaura o topo do stack
               movf    TBLPTRH,w                 ; restaura o topo do stack
               movwf   TOSH                      ; restaura o topo do stack
               movf    TBLPTRL,w                 ; restaura o topo do stack
               movwf   TOSL                      ; restaura o topo do stack
               return                            ; fim

; ---------------------------------------------------------------------------------------------
; 
; ---------------------------------------------------------------------------------------------
_lcdsend_str_c
_lcdsendc      tblrd   *+                        ; le um caracter na lista
	           movf    TABLAT,w                  ; pega o caracter lido
	           bz      _lcdfimstr                ; termina se for nulo
               call    SR_SENDW                  ; senao, imprime o caracter
               bra     _lcdsendc                 ; continua ate encontrar um nulo
_lcdfimstr     return                         ; fim

  ENDIF




; ---------------------------------------------------------------------------------------------
; MACRO: LCD_LINE                                                      Spinelli - 25/05/2011
;
; Monta a tabela de enderecos de linhas de LCD 2x16, 4x16 e 4x20.
; SUPORTA FAMILIAS 16F e 18F
;
; ATENCAO:
; Esta tabela deve residir na mesma pagina que a subrotina 'lcdsr44780.asm'.
;
; CHAMADA:
;              LCD_LINES 0216      ; onde 0216 e o tipo de LCD (2 linha por 16 colunas)
;
; ---------------------------------------------------------------------------------------------
LCD_LINES    MACRO  _LCDTIPO
             local  _LTIPO = _LCDTIPO
   
; ---------------------------------------------------------------------------------------------
;     CODIGO PARA PIC18F4520
; ---------------------------------------------------------------------------------------------
      IF __FAMILIA==18

        IF _LTIPO == 0216
            variable _LCDLEN = .16
LCD_ADDRTAB DB  0x00,0x40
        ENDIF

        IF _LTIPO == 0416
            variable _LCDLEN = .16
LCD_ADDRTAB DB  0x00,0x40,0x90,0xD0         
        ENDIF

        IF _LTIPO == 0420
            variable _LCDLEN = .20
LCD_ADDRTAB DB  0x00,0x40,0x14,0x54
        ENDIF

      ENDIF





; ---------------------------------------------------------------------------------------------
;     CODIGO PARA FAMILIA 12F e 16F
; ---------------------------------------------------------------------------------------------
      IF __FAMILIA<=16
        IF _LTIPO == 0216
            variable _LCDLEN = .16
LCD_ADDRTAB addwf    PCL,f
            DT  0x00,0x40
        ENDIF

        IF _LTIPO == 0416
            variable _LCDLEN = .16
LCD_ADDRTAB addwf    PCL,f
            DT  0x00,0x40,0x90,0xD0          
        ENDIF

        IF _LTIPO == 0420
            variable _LCDLEN = .20
LCD_ADDRTAB addwf    PCL,f
            DT  0x00,0x40,0x14,0x54
        ENDIF

      ENDIF
             ENDM





; -------------------------------------------------------------------------------------
; SET_CURSOR                                                      Spinelli - 25/01/2012
;
; Posiciona o cursor na linha e coluna especificadas.
;
; Esta macro da suporte para as subrotinas de LCD Serial (lcdsr44780) e LCD Paralelo
; (lcdhd44780).
;
; Esta macro chama a funcao SR_SETCURSOR da subrotina "lcdsr44780.asm".
;
; CHAMADA:   SET_CURSOR    linha, coluna, tipo
;
; Onde:      linha ................ numero da linha do LCD
;            coluna ............... numero da coluna do LCD
;            tipo ................. "r" ou "R" linha e coluna estao em registradores
;                                   "i" ou "I" linha e coluna estao inline na macro
;
; EXEMPLO:   SET_CURSOR  .2, .10, "i"
;
; -------------------------------------------------------------------------------------
SET_CURSOR   MACRO    line,col,type
   

   ; ---------------------------------------------------------------------------------- 
   ; Codigo para LCD paralelo - 24/07/2014
   ; ---------------------------------------------------------------------------------- 
   ifdef _PARALLEL_LCD
             IF  (type=="R") || (type=="r")
                 movf   line,w
                 movwf  LCD_R0
                 decf   LCD_R0,f
                 movf   col,w
                 movwf  LCD_R1
                 decf   LCD_R1,f
             ELSE
                 movlw  line 
                 movwf  LCD_R0
                 decf   LCD_R0,f
                 movlw  col 
                 movwf  LCD_R1
                 decf   LCD_R1,f
             ENDIF
   
   ; ---------------------------------------------------------------------------------- 
   ; Codigo para LCD serial com Shift Register - 24/07/2014
   ; ---------------------------------------------------------------------------------- 
   else
             IF  (type=="R") || (type=="r")
                 movf   line,w
                 movwf  SR_1
                 decf   SR_1,f
                 movf   col,w
                 movwf  SR_2
                 decf   SR_2,f
             ELSE
                 movlw  line 
                 movwf  SR_1
                 decf   SR_1,f
                 movlw  col
                 movwf  SR_2
                 decf   SR_2,f
             ENDIF

   endif 

                 call   SR_SETCURSOR

             ENDM  







; ---------------------------------------------------------------------------------------------
; SR_4MVC - move uma string para o LCD.
;
; Esta macro pode ser utilizada com LCD serial (shift register) ou conectado normalmente 
; em modo 4 bits.
;
;
; EXEMPLO PARA A FAMILA 18F
; -------------------------  
; msg001    SR_4MVC  ms001
; ms001     DB       "mensagem",0
;           .
;           .
;           call     msg001
;
;
; EXEMPLO PARA A FAMILIA 16F
; --------------------------
; msg001    ADDWF  PCL,f
;           DT     "mensagem",0
;           .
;           .
;           call     msg001
;  
; ---------------------------------------------------------------------------------------------
SR_4MVC      MACRO    str
_numq        set      _numq + 1
; ---------------------------------------------------------------------------------------------
;            CODIGO PARA PIC18F
; ---------------------------------------------------------------------------------------------
 IF __FAMILIA==18
             movlw   UPPER(str)
             movwf   TBLPTRU
             movlw   HIGH(str)
             movwf   TBLPTRH
             movlw   LOW(str)
             movwf   TBLPTRL
             call    SR_4MVC#v(_numq)
             return
       IFNDEF  SR_4MVC_INDICACOR
             variable  SR_4MVC_INDICACOR
       ENDIF
SR_4MVC#v(_numq) tblrd   *+
             movf    TABLAT,w
             xorlw   0x00
             btfsc   STATUS,Z
             return
             call    SR_SENDW
             goto    SR_4MVC#v(_numq)
     ENDIF

; ---------------------------------------------------------------------------------------------
;            CODIGO PARA PIC16
; ---------------------------------------------------------------------------------------------
 IF __FAMILIA<=16
   IFDEF _LCD_SERIAL
             clrf     LCD_R2                     ; 25/07/2014
S4#v(_numq)  movf     LCD_R2,w                   ; 25/07/2014
   ELSE
             clrf     SR_3                       ; 06/10/2013
S4#v(_numq)  movf     SR_3,w                     ; 06/10/2013
   ENDIF
             call     str
             xorlw    0x00
             btfsc    STATUS,Z
             return
             call     SR_SENDW                   ; 06/10/2013
   IFDEF _LCD_SERIAL
             incf     LCD_R2,f                   ; 25/07/2014
   ELSE
             incf     SR_3  ,f                   ; 06/10/2013
   ENDIF
             goto     S4#v(_numq)
  ENDIF
             ENDM




; ---------------------------------------------------------------------------------------------
; LCD_WRITE - Envia a string inline na macro para o LCD                              04/07/2015
;
; Exemplo:    LCD_WRITE  "Isto e uma mensagem" 
; 
; ---------------------------------------------------------------------------------------------
LCD_WRITE      MACRO   st 
 IF __FAMILIA==18
               call    lcd_sendstr
               dw      st,0x00
 ELSE
   ERROR "Familia nao suportada"
 ENDIF
               ENDM

