; ----------------------------------------------------------------------------------
; Keyboard_services para Familia 18F                                      19/08/2016
;
; STATUS: FUNCIONA
;
;
; Este servico permite ler um teclado comum, PS2 ou DIN5, de PC, e obter o
; Scan Code e o codigo ASCII da tecla pressionada.
;
; Os codigos ASCII dos caracteres do teclado sao mapeados na matriz "SCANCODES_COFF"
; para caracteres minusculos e "SCANCODES_CON" para caracteres maiusculos.
;
; Em primeiro lugar, deve-se usar a funcao "ps2kbd_init" para inicializar as 
; variaveis e as portas do dispositivo PS2 a ser controlado, no caso um teclado PS2.
;
; A funcao "ps2kbd_read" le o teclado e devolve o codigo ASCII do caracter da 
; tecla pressionada.
;
; A funcao "ps2kbd_getscan" le o teclado e retorna o Scan Code da tecla pressionada.
;
; A funcao "ps2kbd_trans" pode ser usada para traduzir um Scan Code para o valor 
; ASCII representado pela tecla pressionada. 
;
; ----------------------------------------------------------------------------------
; DEFINIR OS REGISTRADORES E SIMBOLOS ABAIXO NO PROGRAMA:
;
;ps2kbdTRIS      equ          TRISA         ; I/O do teclado
;ps2kbdLAT       equ          LATA          ; Latch do teclado
;ps2kbdPORT      equ          PORTA         ; Porta do teclaco
;ps2kbdDATA      equ          2             ; bit data
;ps2kbdCLOCK     equ          3             ; bit clock
;ps2flags        res          1             ; flags do servico
;ps2caps         equ          0             ; caps bit --> b'00000000'=OFF   b'00000001'=ON
;ps2capsmsk      equ          1             ; mascara ---> b'00000001'
;ps2ct           res          1             ; contador de bits
;ps2cpy          res          1             ; 
;ps2r1           res          1             ; 
;ps2r2           res          1             ; 
;ps2ledslat      res          1             ; latch dos LEDs do teclado
;ps2buff         res          3             ; buffer de leitura

; ----------------------------------------------------------------------------------


; ----------------------------------------------------------------------------------
; FUNCOES
;
;
; ps2kbd_init ................ Configura as portas DATA e CLOCK inibindo a comunicacao
;                              com o dispositivo PS2.
;
;                              Entrada: Nao ha
;
;                              Saida  : DATA=1, CLOCK=0 e flags=0
;
;
;
; ps2kbd_read ................ Le o Scan Code da tecla pressionada num teclado PS2/DIN5
;                              e devolve o caracter ASCII mapeado para o respectivo 
;                              Scan Code.
; 
;                              Entrada: Nao ha
;
;                              Saida  : W = contera' o caracter ASCII mapeado para o 
;                                       Scan Code referente a tecla pressionada.
;
;
;
; ps2kbd_trans ............... Traduz um Scan Code no seu caracter correspondente.
;
;                              Entrada: W contem o Scan Code
;
;                              Saida  : W contera o caracter correspondente ao Scan Code
;
;
;
; ps2kbd_getscan ............. Retorna o Scan Code de uma tecla.
;                              Para traduzir o Scan Code em seu caracter ASCII, use a
;                              fincao "ps2kbd_trans".
;
;                              Entrada: Nao ha
;
;                              Saida  : W contera o Scan Code da tecla pressionada.
;
;
;
; ps2kbd_cmd ................. Envia um comando para o teclado.
;
;                              Entrada: W contem o dado
; 
;                              Saida  : nao ha
;
;
;
; ps2kbd_req2send ............ Coloca o dispositivo PS2 ne estado "request to send".
;
;                              Entrada: Nao ha
;
;                              Saida  : Nao ha
;
;
;
; ps2kbd_getresp ............. Le o codigo de retorno de um comando enviado ao teclado.
;
;                              Entrada: Nao ha
;
;                              Saida  : W contera o codigo de retorno
;
;
;
; ps2kbd_reset ............... Reset do teclado 
;
;                              Entrada: Nao ha
;
;                              Saida  : W = 0xFA --> reset bem sucedido
;
;
;
; ps2kbd_echo ................ Pede um eco ao teclado
;
;                              Entrada: Nao ha
;
;                              Saida  : W = 0xEE --> eco bem sucedido
;
;
;
; ps2kbd_ledson  ............. acende o LED identificado pelo valor do registrador W.
;
;                              Entrada: W contem a identificacao do LED a ser aceso:
;                                       ps2_SCRLKBIT..........  SCROLL LOCK
;                                       ps2_NUMLKBIT..........  NUM LOCK BIT
;                                       ps2_CAPSLKBIT.........  CAPS LOCK BIT
;
;                              Saida  : W contera o return code do teclado: W=0xFA --> OK
;
;
;
; ps2kbd_ledsoff ............. apaga o LED identificado pelo valor do registrador W.
;
;                              Entrada: W contem a identificacao do LED a ser apagado:
;                                       ps2_SCRLKBIT..........  SCROLL LOCK
;                                       ps2_NUMLKBIT..........  NUM LOCK BIT
;                                       ps2_CAPSLKBIT.........  CAPS LOCK BIT
;
;                              Saida  : W contera o return code do teclado: W=0xFA --> OK
;
; ----------------------------------------------------------------------------------

; ----------------------------------------------------------------------------------
; ESQUEMA DO CONECTOR DIN COM 5 PINOS  
;
; Pino    DIN5            PS2
; ------- -----------     --------
;  1      CLOCK           DATA                                
;  2      DATA            Nao usado  
;  3      Nao Usado       GROUND
;  4      GROUND          +5v
;  5      +5v             CLOCK
;  6      Nao existe      Nao usado
; ----------------------------------------------------------------------------------

; ----------------------------------------------------------------------------------
;  FRAME DE TRANSMISSAO SERIAL DO DISPOSITIVO PS2/DIN5 PARA O HOST
;   
;  1 start bit
;  8 data bits .................. least significant bit first.
;  1 parity bit ................. bit de paridade (paridade impar)
;  1 stop bit
;
; ----------------------------------------------------------------------------------

; ----------------------------------------------------------------------------------
; NOTAS
; 1 - As portas CLOCK e DATA sao portas de coletor aberto e devem possuir resistores
;     externos conectados a alimentacao.
;
; 2 - Para levar as portas CLOCK e DATA ao nivel alto, devemos apenas coloca-las em
;     alta impedancia, ou seja, estabelecemos essas portas como entradas.
; ----------------------------------------------------------------------------------


; ---------------------------------------------------------------
; onkey                                                19/08/2016
;
; Macro para uso com o servico de teclado PS2.
; Esta macro verifica a tecla pressionada e desvia para o
; destino.
;
; Exemplo:  movf    buffer, w         ; pega o codigo do caracter
;           ON_KEY  _DEL,apaga        ; se for DEL vai apagar
;
; ---------------------------------------------------------------
onkey    MACRO   key, dest
         xorlw   key
         btfsc   STATUS, Z
         goto    dest 
         ENDM

; ----------------------------------------------------------------------------------
; PORTUGUES-BR-ABNT2 *** Scan Codes das teclas de funcoes e controles do teclado
; ----------------------------------------------------------------------------------
;Simbolo             Mapa  ScanCode  Tecla
;-----------         ----  --------  ------------------
_ESC          equ    0x1B  ;0x76     ; Escape
_F1           equ    0x05  ;0x05     ; F1
_F2           equ    0x06  ;0x06     ; F2
_F3           equ    0x04  ;0x04     ; F3
_F4           equ    0x0C  ;0x0C     ; F4
_F5           equ    0x03  ;0x03     ; F5
_F6           equ    0x0B  ;0x0B     ; F6
_F7           equ    0x83  ;0x83     ; F7
_F8           equ    0x0A  ;0x0A     ; F8
_F9           equ    0x01  ;0x01     ; F9
_F10          equ    0x09  ;0x09     ; F10
_F11          equ    0x78  ;0x78     ; F11
_F12          equ    0x07  ;0x07     ; F12
_SYSREQ       equ    0x96  ;0x7C     ; Print/Screen/SysRq
_SCROLLK      equ    0x95  ;0x7E     ; SCROLL LOCK
_PAUSEBRK     equ    0x94  ;0x77     ; PAUSE/BREAK
_TAB          equ    0x90  ;0x0D     ; TAB
_BKSP         equ    0x93  ;0x66     ; BACKSPACE
_CAPSLOCK     equ    0x58  ;0x58     ; CAPS LOCK
_SHIFTL       equ    0x12  ;0x12     ; SHIFT do lado esquerdo
_SHIFTR       equ    0x59  ;0x59     ; SHIFT do lado direito
_CTRL_L       equ    0x14  ;0x14     ; CTRL do lado esquerdo
_WIN          equ    0x1F  ;0x1F     ; Windows
_ALT          equ    0x11  ;0x11     ; ALT
_ALTGR        equ    0x11  ;0x11     ; ALTGR
_MENU         equ    0x2F  ;0x2F     ; MENU
_CTRL_R       equ    0x14  ;0x14     ; CTRL do lado direito
_HOME         equ    0x97  ;0x6C     ; HOME
_END          equ    0x98  ;0x69     ; END
_INSERT       equ    0x99  ;0x70     ; INSERT
_PAGEUP       equ    0x9A  ;0x7D     ; PAGE UP
_DEL          equ    0x9B  ;0x71     ; DELETE
_PAGEDN       equ    0x9C  ;0x7A     ; PAGE DOWN
_UP           equ    0x9D  ;0x75     ; ARROW UP
_RIGHT        equ    0x9E  ;0x74     ; ARROW RIGHT
_DOWN         equ    0x9F  ;0x72     ; ARROW DOWN
_LEFT         equ    0xA0  ;0x6B     ; ARROW LEFT
_NUMLOCK      equ    0xB0  ;0x77     ; NUM LOCK

; ----------------------------------------------------------------------------------
; Bits de Controle dos LEDs do teclado usados pelo comando 0xED (Set/Reset LEDs) 
; e pelas funcoes "ps2kbs_ledson" e "ps2kbs_ledsoff".
; ----------------------------------------------------------------------------------
ps2_SCRLKBIT  equ    b'00000001'   ; SCROLL LOCK   - Argumento para o comando 0xED
ps2_NUMLKBIT  equ    b'00000010'   ; NUM LOCK BIT  - Argumento para o comando 0xED
ps2_CAPSLKBIT equ    b'00000100'   ; CAPS LOCK BIT - Argumento para o comando 0xED
; ----------------------------------------------------------------------------------
ps2_brkdelay  equ    .100          ; tempo de retardo, em milisegundos, para evitar 
                                   ; que o BREAK CODE seja capturado quando a tecla                    
                                   ; for liberada.
; ----------------------------------------------------------------------------------





; ----------------------------------------------------------------------------------
; ps2kbd_init
;
; Configura as portas DATA e CLOCK inibindo a comunicacao com o dispositivo PS2.
;
; O pino DATA deve se manter em nivel alto e o pino CLOCK em nivel baixo para
; inibir a comunicacao com o dispositivo PS2.
;
; Como as portas do dispositivo PS2 sao do tipo coletor aberto, o pino DATA 
; deve permanecer no estado de alta impedancia (INTUP) pois supoe-se que ha um
; resistor de pullup nessa linha.
;
; O pino CLOCK deve permanecer baixo (OUTPUT) por pelo menos 100um para que o
; dispositivo PS2 entenda que a comunicacao foi inibida.
;
;
; Entrada: Nao ha
;
; Saida  : Nao ha
;
; Chamada:     call    ps2kbd_init
;
;
; NOTAS
; 1 - Apos essa funcao, pode-se usar a funcao "ps2kbd_echo" para verificar se
;     o teclado esta respondendo.
;
; ----------------------------------------------------------------------------------
ps2kbd_init    bcf     ps2kbdLAT,  ps2kbdDATA      ; pino de dado em nivel baixo
               bcf     ps2kbdLAT,  ps2kbdCLOCK     ; pino de clock em nivel baixo

               bsf     ps2kbdTRIS, ps2kbdDATA      ; pino de dado em alta impedancia
               bcf     ps2kbdTRIS, ps2kbdCLOCK     ; pino de clock como saida

               clrf    ps2flags                    ; limpa os flags         
               clrf    ps2ledslat                  ; limpa latch dos LEDs do teclado    
               call    ps2kbd_clrbuf               ; limpa buffer de I/O do teclado
               movlw   .200                        ; 200us
               call    delay_us                    ; delay
               return

; ----------------------------------------------------------------------------------
; ps2kbd_clrbuf - limpa o buffer de leitura
;
; Esta e' uma funcao interna.
; ----------------------------------------------------------------------------------
ps2kbd_clrbuf  clrf    ps2buff+0
               clrf    ps2buff+1
               clrf    ps2buff+2
               return

; ----------------------------------------------------------------------------------
; ps2kbd_getscan                                                          10/08/2016
;
; Retorna o Scan Code de uma tecla
;
; Entrada: Nao ha
;
; Saida  : W contera o Scan Code da tecla pressionada.
;
; Chamada:     call    ps2kbd_getscan            ; le o Scan Code 
;              movwf   scancode                  ; salva o scan code lido
;
; ----------------------------------------------------------------------------------
ps2kbd_getscan call    ps2kbd_read               ; le o teclado
               movf    ps2buff+0, w              ; recupera o Scan Code lido
               return                            ; devolve o Scan Code em W


; --------------------------------------------------------------------------------------------------------------
; ps2kbd_cmd - Envia um comando ao teclado                                                           11/080/2016
;
; Esta funcao envia um comando ou argumento de comando ao teclado.
;
; Antes de usar esta funcao, considere usar as funcoes que ja estao prontas como "ps2kbd_reset",
; "ps2kbd_ledson", "ps2kbd_ledsoff" e "ps2kbd_echo", senao, siga as instrucoes abaixo:
;
; 1 - Antes de enviar um comando, e' necessario colocar o teclado em estado de recepcao com a
;     funcao "ps2kbd_req2send".
;
; 2 - O comando deve ser enviado carregando-se W com o codigo do comando desejado e fazendo uma
;     chamada a funcao "ps2kbd_cmd".
;
; 3 - Se o comando possui um argumento, esse argumento deve ser enviado logo em seguida ao comando.
;     Para enviar o argumento, carrega-lo no registrador e fazer mais uma chamada a funcao "ps2kbd_cmd".
;
; 4 - Apos enviar um comando e, se houver, seu argumento, e' necessario usar a funcao  'ps2kbd_getresp'
;     para obter a resposta do teclado que sera retornada no registrador W. Se W contiver 0xFA, entao
;     o comando foi executado com sucesso. Se W contiver outro valor, um erro ocorreu.
;
;
;
; Entrada: W contem o valor a enviar (comando ou dado).
; 
; Saida  : W contera um codigo de retorno: W = 0xFA --> OK
;
; Chamada:      call    ps2kbd_req2send ; Request to Send
;               movlw   0xFF            ; comando Keyboard Reset
;               call    ps2kbd_cmd      ; envia o comando
;               call    ps2kbd_getresp  ; obtem a resposta do teclado
;               xorlw   0xFA            ; verifica resposta do teclado
;               btfss   STATUS, Z       ; comando foi aceito???
;               goto    erro            ; NAO --> terminou mal!!!
;
; --------------------------------------------------------------------------------------------------------------



; --------------------------------------------------------------------------------------------------------------
; DIAGRAMA DE TEMPO DA COMUNICACAO HOST->TECLADO
;
;                  1       2       3       4       5       6       7       8       9       10      11      12
; CLOCK -----+   +---+   +---+   +---+   +---+   +---+   +---+   +---+   +---+   +---+   +---+   +---+   +-----
;            |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |
;            |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |
;            +---+   +---+   +---+   +---+   +---+   +---+   +---+   +---+   +---+   +---+   +---+   +---+
;              1       2       3       4       5       6       7       8       9       10      11      12
;
; DATA -------+        +-------+-------+-------+-------+-------+-------+-------+-------+------+-----+      +---
;             |        ||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||     |      |
;             |        ||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||     |      |
;             +--------+-------+-------+-------+-------+-------+-------+-------+-------+------+     +------+
;               START     D0      D1      D2      D3      D4      D5      D6      D7    PARITY STOP    ACK
;
; --------------------------------------------------------------------------------------------------------------



; --------------------------------------------------------------------------------------------------------------
; DESCRICAO DO PROTOCOLO
;  
; Se o host deseja se comunicar com o dispositivo PS2, ele deve primeiramente colocar os pinos
; DATA e CLOCK no estado "Request-to-send": DATA=0 e CLOCK=1 (Input);
;
; Quando o teclado PS2 deteta esse estado, comeca a gerar sinais de CLOCK e envia 8 bits de dados e um
; bit de STOP ao host.
;
; O host deve modificar o sinal DATA somente quando o sinal de CLOCK estiver baixo (CLOCK=0).
;
; O dado em DATA sera lido pelo dispositivo PS2 quando o CLOCK estiver alto (CLOCK=1).
;
; Apos o STOP bit ser recebido pelo dispositivo PS2, ele levara o pino DATA ao nivel baixo (DATA=0),
; dizendo que recebeu o data (ACK), e emitira o ultimo pulso de CLOCK.
; Se o host nao liberar o pino DATA (DATA=Input) apos o decimo primeiro pulso de CLOCK, o dispositivo PS2
; continuara a gerar pulsos de CLOCK ate que o pino DATA seja liberado. 
;
; O host pode abortar a transmissao antes do decimo primeiro pulso de CLOCK (ACK bit) fazendo
; CLOCK=0 durante pelo menos 100us (microsegundos).
;
; Os passos abaixo resumem o que foi dito acima:
;
;  1 - CLOCK=0 no minimo por 100us (microsegundos) para inibir a comunicacao.
;  2 - DATA=0 ---------> "Request-to-send" state
;  3 - CLOCK=input ----> "Request-to-send" state - o PS2 nao deve exceder 10ms para responder.
;  4 - Ler o pino CLOCK e esperar que o PS2 responda com CLOCK=0 
;  5 - Colocar o dado no bit DATA
;  6 - Esperar que o PS2 torne o CLOCK=1 
;  7 - Esperar que o PS2 torne o CLOCK=0 
;  8 - Repetir itens 5 a 7 para os 7 bits de dados restantes e para o PARIRY bit. 
;  9 - Fazer bit DATA=input 
; 10 - Esperar o PS2 fazer DATA=0 ----> ACK do teclado
; 11 - Esperar o PS2 fazer CLOCK=0 ---> ACK do teclado
; 12 - Esperar o PS2 fazer DATA=input e CLOCK=intput (liberar Data e Clock)
; --------------------------------------------------------------------------------------------------------------
ps2kbd_cmd     movwf   ps2r1                     ; salva copia do dado a ser enviado
               clrf    ps2buff+0                 ; usado como registrador auxiliar para calculo do PARITY bit 
               clrf    ps2buff+1                 ; usado como registrador auxiliar para calculo do PARITY bit 

               movlw   .8                        ; SIM --> numero de bits a enviar
               movwf   ps2ct                     ; carrega contador de bits

               ; PASSO 4 (Wait CLOCK=0)
               call    ps2_wclkdown              ; wait CLOCK=0


               ; PASSOS 5, 6 e 7
ps2_req999     call    ps2_req567                ; passos 5, 6 e 7

               ; PASSO 8
               decfsz  ps2ct, f                  ; conta bit enviado
               goto    ps2_req999                ; repete ate enviar 8 bits

               ; Envia o PARITY bit
               ; Se o numero de bits presentes no byte enviado for par, o parity bit sera 1.
               ; Se o numero de bits presentes no byte enviado for impar, o parity bit sera 0.
               clrf    ps2r1                     ; para um numero impar de bits ==> PARITY bit = 0 
               rrcf    ps2buff+0, f              ; LSB -> C  para saber se par ou impar
               btfss   STATUS, C                 ; era um numero impar de bits???
               bsf     ps2r1, 0                  ; NAO --> PARITY bit = 1 ==> era m numero par de bits
               call    ps2_req567                ; envia o PARITY bit

               ; PASSO 9 (DATA=Input - release DATA)
               bcf     ps2kbdLAT,  ps2kbdDATA    ; DATA=0
               bsf     ps2kbdTRIS, ps2kbdDATA    ; DATA ==> Input
               ; PASSO 10 - Espera ACK do teclado -> DATA=0 
               call    ps2_wdatdown              ; wait DATA=0

               ; PASSO 11 - Espera ACK do teclado -> CLOCK=0 
               call    ps2_wclkdown              ; wait CLOCK=0

               call    ps2_wclkup
               call    ps2_wdatup
               bcf     ps2kbdTRIS, ps2kbdDATA    ; DATA=OUTPUT
               return                            ; fim da funcao ps2kbd_cmd 

; ----------------------------------------------------------------------------------
; ps2kbd_read                                                             05/08/2016
;
; Le o Scan Code da tecla pressionada num teclado PS2/DIN5 e devolve o caracter
; ASCII mapeado para o respectivo Scan Code.
;
; As teclas de funcoes F1 a F12 e as teclas de controle como Tab, CapsLock, Shift, Ctrl, etc
; serao mapeadas com seus proprios Scan Codes, isto e', nao terao traducao em ASCII.
; 
; Entrada: Nao ha
;
; Saida  : W = contera' o caracter ASCII mapeado para o Scan Code referente a
;              tecla pressionada.
;         
; Chamada:     call   ps2kbd_read         ; le o teclado
;              movwf  tecla               ; salva o codigo ASCII 
;
; ----------------------------------------------------------------------------------
; COMUNICACAO DO TECLADO PARA O HOST
;
; Extraido e adaptado de http://www.computer-engineering.org/ps2protocol/
;
; Para receber dados do dispositivo PS2, o host deve colocar o pino Clock e Data em nivel alto e esperar que
; o dispositivo PS2 responda com um START bit levando o pino DATA ao nivel baixo. Dai em diante, o dispositivo
; PS2 vai controlar a transmissao, colocando bits de dados no pino DATA e baixando e lenvantando o sinal de CLOCK.
; O pino DATA muda de estado enquanto o pino CLOCK esta em nivel alto.
; O dado pode ser lido pelo host no pino DATA enquanto o pino CLOCK estiver baixo.  
;
;  ESTADOS   1    2   3    2   3     2   3     2   3     2   3     2   3     2   3     2   3     2   3     2   3     2   1
;  CLOCK -------+   +----+   +-----+   +-----+   +-----+   +-----+   +-----+   +-----+   +-----+   +-----+   +-----+   +-----------
;               |   |    |   |     |   |     |   |     |   |     |   |     |   |     |   |     |   |     |   |     |   |
;               |   |    |   |     |   |     |   |     |   |     |   |     |   |     |   |     |   |     |   |     |   |
;               +---+    +---+     +---+     +---+     +---+     +---+     +---+     +---+     +---+     +---+     +---+
;
;  DATA ------+       +---------+---------+---------+---------+---------+---------+---------+---------+------+---------------------
;             |       ||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
;             |       ||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
;             +-------+---------+---------+---------+---------+---------+---------+---------+---------+------+
;               START    DATA     DATA      DATA       DATA     DATA      DATA      DATA       DATA    PARITY      STOP 
;                BIT      0        1         2         3        4         5         6          7                   BIT 
;
; ----------------------------------------------------------------------------------

ps2kbd_read    call    ps2kbd_clrbuf              ; limpa o buffer de leitura

               ; Coloca as portas CLOCK e DATA em IDLE STATE para o dispositivo PS2 poder 
               ; iniciar uma transmissao.
               ; As portas CLOCK e DATA sao portas de coletor aberto e devem possuir resistores
               ; externos conectados a alimentacao, assim, para levar essas portas ao 
               ; nivel alto, apenas as colocamos em alta impedancia, ou seja, estabelecemos essas
               ; portas como entradas.
               bsf     ps2kbdTRIS, ps2kbdDATA     ; pino de DATA  em alta impedancia
               bsf     ps2kbdTRIS, ps2kbdCLOCK    ; pino de CLOCK em alta impedancia

ps2kbdrst      movlw   .11                        ; numero de bits esperados na transmissao
               movwf   ps2ct                      ; carrega contador de bits

               ; Aguarda chegar um START bit (espera o operador pressionar uma tecla)
ps2wstart      call     ps2_wdatdown              ; wait DATA=0

ps2wclkdw1     call     ps2_wclkdown              ; wait CLOCK=0
               movf    ps2kbdPORT, w              ; le a porta de dados
               movwf   ps2cpy                     ; salva uma copia da leitura
               bcf     STATUS, C                  ; default Carry=0
               btfsc   ps2cpy, ps2kbdDATA         ; DATA==0 ???
               bsf     STATUS, C                  ; NAO --> Carry=1
               rrcf    ps2buff+0, f               ; C -> buffer+0 MSB   buffer+0 LSB -> C
               rrcf    ps2buff+1, f               ; C -> buffer+1 MSB   buffer+1 LSB -> C
               rrcf    ps2buff+2, f               ; C -> buffer+2 MSB   buffer+2 LSB -> C
               decfsz  ps2ct, f                   ; fim dos dados ???
               goto    ps2wclkup1                 ; NAO --> vai esperar descida do clock
               goto    ps2complete                ; SIM --> 
ps2wclkup1     call    ps2_wclkup                 ; wait CLOCK=1
               goto    ps2wclkdw1                 ; vai esperar descida do clock

ps2complete    ; Processa o STOP bit
               ; Quando sair deste loop, estara' em STOP
               call    ps2_wclkup                 ; wait CLOCK=1
               call    ps2_wdatup                 ; wait DATA=1

               ; Vejamos se o estado de STOP continua por algum tempo
               clrf    ps2ct                      ; timeout = 256
               movlw   .100                       ; timeout = 256 * 100 = 25600 ciclos
               movwf   ps2r1

               ; O fim da transmissao deve se caracterizar por um STOP bit (DATA=1 e CLOCK=1).
               ; Este estado deve permanecer por mais de 71ms quando o operador libera a tecla
               ; que estava pressionada. 
               ; Este segmento dura 358400 ciclos (14 * 25600 = 358400), que correspondem a 
               ; 71,68ms (milisegundos) com Fosc de 20MHZ, que e' o suficiente para saber se 
               ; o operador liberou a tecla.
               ; Quando a tecla e' liberada, o teclado envia um BREAK de ate 3 bytes, fazendo
               ; com que o STOP bit dure muito menos do que os 71ms que caracterizam uma pausa
               ; longa na digitacao. Neste caso, esta funcao lera os proximos bits enviados
               ; e armazena-los-a no buffer. 
                                                  ;                                CICLOS  TOTAL
ps2procstop    movf    ps2kbdPORT, w              ; le a porta de dados               1
               movwf   ps2cpy                     ; salva uma copia da leitura        1 ->  2
               btfss   ps2cpy, ps2kbdCLOCK        ; CLOCK==1 ???                      1 ->  3
               goto    ps2kbdrst                  ; NAO --> saiu de STOP              2 ->  5
               btfss   ps2cpy, ps2kbdDATA         ; DATA==1 ???                       1 ->  6
               goto    ps2kbdrst                  ; NAO --> saiu de STOP              2 ->  8
               decfsz  ps2ct, f                   ;                                   1 ->  9
               goto    ps2procstop                ;                                   2 -> 11 
               decfsz  ps2r1, f                   ;                                   1 -> 12 
               goto    ps2procstop                ;                                   2 -> 14 

               ; Desloca o Scan Code para o primeiro byte do buffer de leitura
               rlcf    ps2buff+2, f
               rlcf    ps2buff+1, f
               rlcf    ps2buff+0, f
               rlcf    ps2buff+2, f
               rlcf    ps2buff+1, f
               rlcf    ps2buff+0, f

               ; Verifica se a tecla Caps Lock (Scan Code 0x58) foi pressionada
               ; invertendo o flag correspondente deste estado.
               movlw   _CAPSLOCK               ; CAPS_LOCK Key Scan Code
               xorwf   ps2buff+0, w               ; verifica se e' CAPS_LOCK
               btfss   STATUS, Z                  ; CAPS_LOCK pressionado???
               goto    ps2vsc12                   ; verifica 
               movlw   ps2capsmsk                 ; mascara para inversao
               xorwf   ps2flags, f                ; SIM --> inverte CAPS

               btfss   ps2flags, ps2caps 
               goto    ps2stcof
               movlw   ps2_CAPSLKBIT
               call    ps2kbd_ledson
               retlw   _CAPSLOCK
ps2stcof       movlw   ps2_CAPSLKBIT
               call    ps2kbd_ledsoff
               movlw   ps2_brkdelay               ; tempo de retardo
               call    delay_ms                   ; retardo para nao capturar BREAK CODE
               retlw   _CAPSLOCK 


               ; Verifica se a tecla SHIFT do lado esquerdo do teclado esta pressionada
ps2vsc12       movlw   _SHIFTL                ; SHIFT (lado esquerdo)
               xorwf   ps2buff+0, w               ; verifica se e' CAPS_LOCK
               btfss   STATUS, Z                  ; CAPS_LOCK pressionado???
               goto    ps2vsc59                   ; verifica 
               movlw   ps2capsmsk                 ; mascara para inversao
               xorwf   ps2flags, f                ; SIM --> inverte CAPS
               goto    ps2kbdrst                  
 
               ; Verifica se a tecla SHIFT do lado direito do teclado esta pressionada
ps2vsc59       movlw   _SHIFTR                ; SHIFT (lado direito)
               xorwf   ps2buff+0, w               ; verifica se e' CAPS_LOCK
               btfss   STATUS, Z                  ; CAPS_LOCK pressionado???
               goto    ps2vnext                   ; verifica 
               movlw   ps2capsmsk                 ; mascara para inversao
               xorwf   ps2flags, f                ; SIM --> inverte CAPS
               goto    ps2kbdrst                  
 
               ; Converte um Scan Code para o caracter ASCII mapeado
               ; e devolve ao chamador no registrador W
ps2vnext       movlw   ps2_brkdelay               ; tempo de retardo
               call    delay_ms                   ; retardo para nao capturar BREAK CODE
               movf    ps2buff+0, w               ; pega o Scan Code
               call    ps2kbd_trans               ; vai converter
               return                             ; devolve o caracter ao chamador


; ----------------------------------------------------------------------------------
; ps2kbd_trans                                                            08/08/2016
;
; Traduz um Scan Code no seu caracter ASCII correspondente.
;
; Entrada: W contem o Scan Code
;
; Saida  : W contera o caracter correspondente ao Scan Code
;
; Chamada:     call    ps2kbd_getscan            ; le o Scan Code - W contera o Scan Code
;              call    ps2kbd_trans              ; faz a traducao para ASCII
;              movwf   letra                     ; salva 
;
; ----------------------------------------------------------------------------------
ps2kbd_trans   movwf   ps2cpy                    ; salva o Scan Code
               btfsc   ps2flags, ps2caps
               goto    ps2kbdcon
               movlw   UPPER(SCANCODES_COFF)
               movwf   TBLPTRU
               movlw   HIGH(SCANCODES_COFF)
               movwf   TBLPTRH
               movlw   LOW(SCANCODES_COFF)
               movwf   TBLPTRL
               goto    ps2resume
ps2kbdcon      movlw   UPPER(SCANCODES_CON)
               movwf   TBLPTRU
               movlw   HIGH(SCANCODES_CON)
               movwf   TBLPTRH
               movlw   LOW(SCANCODES_CON)
               movwf   TBLPTRL
ps2resume      movf    ps2cpy, w
               addwf   TBLPTRL, f
               btfss   STATUS, C
               goto    ps2kbdtr1
               infsnz  TBLPTRH, f
               incf    TBLPTRU, f
ps2kbdtr1      tblrd   *+
               movf    TABLAT, w
               return

; --------------------------------------------------------------------------------------------
; ps2kbd_reset - Reset do teclado                                                   15/08/2016
;
; Entrada: Nao ha
;
; Saida  : W = 0xFA --> reset bem sucedido
;
; Chamada:     call    ps2kbd_reset      ; envia 'Reset'
;              xorlw   0xFA              ; verifica resposta
;              btfss   STATUS, Z         ; teclado reconfigurado???
;              goto    erro              ; NAO --> erro 
;
; --------------------------------------------------------------------------------------------
ps2kbd_reset   call    ps2kbd_req2send           ; request to send
               movlw   0xFF                      ; comando 'Reset'
               call    ps2kbd_cmd                ; envia o comando
               call    ps2kbd_getresp            ; espera a resposta
               clrf    ps2ledslat                ; limpa latch do LEDS
               clrf    ps2flags                  ; limpa os flags
               return

; --------------------------------------------------------------------------------------------
; ps2kbd_echo - Pede um eco ao teclado                                              14/08/2016
;
; Entrada: Nao ha
;
; Saida  : W = 0xEE --> eco bem sucedido
;
; Chamada:     call    ps2kbd_echo       ; pede um 'Echo'        
;              xorlw   0xEE              ; verifica resposta
;              btfss   STATUS, Z         ; 'Echo' chegou???
;              goto    erro              ; NAO --> erro
;
;
; --------------------------------------------------------------------------------------------
ps2kbd_echo    call    ps2kbd_req2send           ; request to send
               movlw   0xEE                      ; comando 'Echo'
               call    ps2kbd_cmd                ; envia o comando
               call    ps2kbd_getresp            ; espera a resposta
               return



; --------------------------------------------------------------------------------------------



; -------------------------------------------------------------------------------
; Rotinas de Wait para os pino DATA e CLOCK
;
; Rotinas internas usadas pela funcao 'ps2kbd_cmd'
;
; ------------------------------------------------------------------------------
ps2_wdatdown   movf    ps2kbdPORT, w
               movwf   ps2cpy
               btfsc   ps2cpy, ps2kbdDATA        ; DATA==0???
               goto    ps2_wdatdown              ; NAO --> espera DATA=0
               return

ps2_wdatup     movf    ps2kbdPORT, w
               movwf   ps2cpy
               btfss   ps2cpy, ps2kbdDATA        ; DATA==0???
               goto    ps2_wdatup                ; NAO --> espera DATA=1
               return 

ps2_wclkdown   movf    ps2kbdPORT, w
               movwf   ps2cpy
               btfsc   ps2cpy, ps2kbdCLOCK       ; DATA==0???
               goto    ps2_wclkdown              ; NAO --> espera CLOCK=0
               return

ps2_wclkup     movf    ps2kbdPORT, w
               movwf   ps2cpy
               btfss   ps2cpy, ps2kbdCLOCK       ; DATA==0???
               goto    ps2_wclkup                ; NAO --> espera CLOCK=1
               return

; -------------------------------------------------------------------------------
; Envia 1 bit ao dispositivo PS2 e calcula o PARITYBIT.
;
; Rotina interna usada pela funcao 'ps2kbd_cmd'
;
; ------------------------------------------------------------------------------
ps2_req567     ; PASSO 5
               bcf     ps2kbdLAT, ps2kbdDATA       ; DATA=0
               rrcf     ps2r1, f                    ; LSB -> C
               btfsc   STATUS, C                   ; C==0???
               bsf     ps2kbdLAT, ps2kbdDATA       ; NAO --> DATA=1 

               ; CALCULO PARITY BIT --> PARIDADE IMPAR
               ; Se o numero de bits presentes no byte enviado for par, o parity bit sera 1.
               ; Se o numero de bits presentes no byte enviado for impar, o parity bit sera 0.
               rlcf    ps2buff+1, f                ; buff+1 = .... ...x <- C
               movlw   0x01                        ;      W = 0000 0001
               andwf   ps2buff+1, w                ;      W = 0000 000y
               addwf   ps2buff+0, f                ; buff+0 = 0000 SSSS --> soma
     
               ; PASSO 6
               call    ps2_wclkup                  ; wait CLOCK=1

               ; PASSO 7
               call    ps2_wclkdown                ; wait CLOCK=0

               return



; -------------------------------------------------------------------------------
; ps2kbd_req2send - REQUEST TO SEND
;
; Coloca o dispositivo PS2 ne estado "request to send".
;
; Entrada: Nao ha
;
; Saida  : O pino DATA estara em nivel baixo (OUTPUT).
;          O pino CLOCK sera colocado em alta impedancia (INPUT).
;
; Chamada:    call    ps2kbd_req2send
;
; ------------------------------------------------------------------------------
               ; PASSO 1, 2 e 3
ps2kbd_req2send 
               bcf     ps2kbdLAT,  ps2kbdCLOCK   ; CLOCK=0
               bcf     ps2kbdTRIS, ps2kbdCLOCK   ; CLOCK ==> Output
               movlw   .60                       ; 
               call    delay_us                  ; delay

               ; PASSO 2 ( Send START bit)
               bcf     ps2kbdLAT,  ps2kbdDATA    ; DATA=0
               bcf     ps2kbdTRIS, ps2kbdDATA    ; DATA ==> Output
               movlw   .60                       ; 
               call    delay_us                  ; delay

               ; PASSO 3 (Release CLOCK)
               bsf     ps2kbdLAT, ps2kbdCLOCK 
               bsf     ps2kbdTRIS, ps2kbdCLOCK     ; CLOCK ==> Output (Release CLOCK)
               return


; -------------------------------------------------------------------------------
; ps2kbd_getresp - GET RESPONSE CODE
;
; Le o codigo de retorno de um comando enviado ao teclado.
;
; Entrada: Nao ha
;
; Saida  : W contera o codigo de retorno.
;
; Chamada:     movlw    0xFF
;              call     ps2kbd_cmd
;              call     ps2kbd_getresp
;              xorlw    0xFA
;              btfss    STATUS, Z
;              goto     erro
;
; ------------------------------------------------------------------------------
ps2kbd_getresp movlw   .11
               movwf   ps2ct
               bcf     ps2kbdLAT,  ps2kbdDATA    ; DATA=0
               bsf     ps2kbdTRIS, ps2kbdDATA    ; DATA ==> Input
ps2kbd_resp01  call    ps2_wclkdown
               movf    ps2kbdPORT, w
               movwf   ps2cpy
               bcf     STATUS, C
               btfsc   ps2cpy, ps2kbdDATA
               bsf     STATUS, C
               rrcf     ps2buff+0, f
               rrcf     ps2buff+1, f
               call    ps2_wclkup
               decfsz  ps2ct, f
               goto    ps2kbd_resp01
               rlcf     ps2buff+1, f
               rlcf     ps2buff+0, f
               rlcf     ps2buff+1, f
               rlcf     ps2buff+0, f
               movf    ps2buff+0, w
               return


; --------------------------------------------------------------------------------------------
; ps2kbd_ledsoff  - apaga o LED identificado pelo valor do registrador W.           13/08/2016
;
; Entrada: W contem a identificacao do LED a ser apagado:
;             ps2_SCRLKBIT..........  SCROLL LOCK
;             ps2_NUMLKBIT..........  NUM LOCK BIT
;             ps2_CAPSLKBIT.........  CAPS LOCK BIT
;
; Saida  : W contera o return code do teclado: W=0xFA --> OK
;
; Chamada:    movlw   ps2_CAPSLKBIT     ; requisita Caps Lock
;             call    ps2kbd_ledson     ; acende Caps Lock
;
; --------------------------------------------------------------------------------------------
ps2kbd_ledsoff movwf   ps2r2                     ; salva ID do LED
               call    ps2kbd_req2send           ; request to send
               movlw   0xED                      ; comando
               call    ps2kbd_cmd                ; envia o comando
               comf    ps2r2, w                  ; carrega ID invertido do LED
               andwf   ps2ledslat, f             ; apaga o LED no latch
               movf    ps2ledslat, w             ; pega o latch
               call    ps2kbd_cmd                ; envia o argumento
               call    ps2kbd_getresp            ; espera a resposta
               return

ps2kbd_ledson  movwf   ps2r2                     ; salva ID do LED  .... .x..
               call    ps2kbd_req2send           ; request to send
               movlw   0xED                      ; comando
               call    ps2kbd_cmd                ; envia o comando
               movf    ps2r2, w                  ; carrega ID do LED
               iorwf   ps2ledslat, f             ; acende o LED no latch
               movf    ps2ledslat, w             ; pega o latch
               call    ps2kbd_cmd                ; envia o argumento
               call    ps2kbd_getresp            ; espera a resposta
               return



; --------------------------------------------------------------------------------------------
; PS2 PORTUGUES BR ABNT2_KEYBOARD SCAN CODES TO CHARACTER TABLE FOR CAPS_ON 
; --------------------------------------------------------------------------------------------  
SCANCODES_CON
;      0    1    2    3    4    5    6    7    8    9    A    B    C    D    E     F   
;           F9        F5   F3   F1   F2   F12       F10  F8   F6   F4  TAB   '
 DB   0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x07,0x08,0x09,0x0A,0x0B,0x0C,0x0D,0x27,0x0F    ; 0

;                                Q    !                   Z    S    A   W     @
 DB   0x10,0x11,0x12,0x13,0x14,0x51,0x21,0x17,0x18,0x19,0x5A,0x53,0x41,0x57,0x40,0x1F    ; 1

;            C    X    D    E    $    #             spc   V    F    T    R    %
 DB   0x20,0x43,0x58,0x44,0x45,0x24,0x23,0x27,0x28,0x20,0x56,0x46,0x54,0x52,0x25,0x2F    ; 2

;            N    B    H    G    Y                       M    J    U    &    *
 DB   0x30,0x4E,0x42,0x48,0x47,0x59,0xA8,0x37,0x38,0x39,0x4D,0x4A,0x55,0x26,0x2A,0x3F    ; 3

;            <    K    I    O    )    (              >    :    L         P    _
 DB   0x40,0x3C,0x4B,0x49,0x4F,0x29,0x28,0x47,0x48,0x3E,0x3A,0x4C,0x4C,0x50,0x5F,0x4F    ; 4

;            ?    ^         `    +            CAPL SFD  ENTR   {         }
 DB   0x50,0x3F,0x5E,0x53,0x60,0x2B,0x56,0x57,0x58,0x59,0x0D,0x7B,0x00,0x7D,0x5E,0x5F    ; 5

;            |                      BKSP           END         4  HOME
 DB   0x60,0x7C,0x62,0x63,0x64,0x65,0x93,0x67,0x68,0x98,0x6A,0xA0,0x97,0x6D,0x6E,0x6F    ; 6

;      0   DEL   2    5    6    8   ESC  NMLK       +    3    -   SYSR  9   SRLL
 DB   0x70,0x9B,0x9F,0x73,0x9E,0x9D,0x1B,0x94,0x78,0x79,0x7A,0x7B,0x96,0x7D,0x95,0x7F    ; 7

;                     F7
 DB   0x80,0x81,0x82,0x83,0x84,0x85,0x86,0x87,0x88,0x89,0x8A,0x8B,0x8C,0x8D,0x8E,0x8F    ; 8

; --------------------------------------------------------------------------------------------


; --------------------------------------------------------------------------------------------
; PS2 PORTUGUES BR ABNT2_KEYBOARD SCAN CODES TO CHARACTER TABLE FOR CAPS_OFF
; --------------------------------------------------------------------------------------------  
SCANCODES_COFF
;      0    1    2    3    4    5    6    7    8    9    A    B    C    D    E     F   
;           F9        F5   F3   F1   F2   F12       F10  F8   F6   F4  TAB   "
 DB   0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x07,0x08,0x09,0x0A,0x0B,0x0C,0x0D,0x22,0x0F    ; 0

;          ALT  SFL       CTRL   q    1                   z    s    a   w     2  WIN 
 DB   0x10,0x11,0x12,0x13,0x14,0x71,0x31,0x17,0x18,0x19,0x7A,0x73,0x61,0x77,0x32,0x1F    ; 1

;            c    x    d    e    4    3             spc   v    f    t    r    5  MENU
 DB   0x20,0x63,0x78,0x64,0x65,0x34,0x33,0x27,0x28,0x20,0x76,0x66,0x74,0x72,0x35,0x2F    ; 2

;            n    b    h    g    y    6                   m    j    u    7    8
 DB   0x30,0x6E,0x62,0x68,0x67,0x79,0x36,0x37,0x38,0x39,0x6D,0x6A,0x75,0x37,0x38,0x3F    ; 3

;            ,    k    i    o    0    9              .    ;    l         p    -
 DB   0x40,0x2C,0x6B,0x69,0x6F,0x30,0x39,0x47,0x48,0x2E,0x3B,0x6C,0x00,0x70,0x2D,0x4F    ; 4

;            /    ~             =            CAPL SFD  ENTR   [         ]
 DB   0x50,0x2F,0x7E,0x53,0xB4,0x3D,0x56,0x57,0x58,0x59,0x0D,0x5B,0x5C,0x5D,0x5E,0x5F    ; 5

;            \                      BKSP           END         4  HOME   .
 DB   0x60,0x5C,0x62,0x63,0x64,0x65,0x93,0x67,0x68,0x98,0x6A,0xA0,0x97,0x6D,0x6E,0x6F    ; 6

;      0   DEL    2    5    6    8  ESC  NMLK F11    +    3    -  SYSR   9  SRLL
 DB   0x70,0x9B,0x9F,0x73,0x9E,0x9D,0x1B,0x94,0x78,0x79,0x7A,0x7B,0x96,0x7D,0x95,0x7F    ; 7

;                     F7
 DB   0x80,0x81,0x82,0x83,0x84,0x85,0x86,0x87,0x88,0x89,0x8A,0x8B,0x8C,0x8D,0x8E,0x8F    ; 8

