; ---------------------------------------------------------------------- 
; TECLADO PS2 PARA FAMILIA 18F                                19/08/2016
;
; STATUS: FUNCIONA
;
;
; ----------------------------------------------------------------
; FUNCOES DE TECLAS
;
; Caps Lock ............ Trava/destrava o teclado em maiusculas/minusculas.
; Del .................. Apaga o LCD
; F1 ................... Habilita/Desabilita funcao de leitura de Scan Codes
; F12 .................. Pisca todos os LEDs do teclado.
; BackSpace ............ Retrocede o cursor do LCD apagando o caracter sobre ele.
; Home ................. Move o cursor para a coluna 1 da linha 1.
; End .................. Move o cursor para o fim da linha corrente.
; Seta a direita ....... Move o cursor para a proxima coluna.
; Seta a esquerda ...... Move o cursor para a coluna anterior.
; Seta para baixo ...... Move o cursor para a proxima linha.
; Seta para cima ....... Move o cursor para a linha anterior.
; Print/Screen/Sys Rq .. Reboot no microcontrolador.
;
; ----------------------------------------------------------------------

; ----------------------------------------------------------------------
; HARDWARE
; Fosc ............ 20MHz externo
; RA2 ............. LCD DataLCD Clock
; RA3 ............. LCD Clock
; RB3 ............. Keyboard Clock
; RB4 ............. Keyboard Data
;
; ----------------------------------------------------------------------

	LIST P=18F4520
	#include <P18F4520.INC>
    errorlevel -302
;
; ----------------------------------------------------------------------
; CONFIGURATION BITS ou FUSES
; -----------------------------------------------------------------------
   CONFIG WDT    = OFF
   CONFIG MCLRE  = ON
   CONFIG DEBUG  = OFF
   CONFIG LVP    = OFF 
   CONFIG OSC    = HS       

             #include  "familias.inc"

             UDATA  0x080

kbdchar      res    .1
RL           res    .1
RC           res    .1
delay_temp   RES    .1
CLOCK        equ   20000000
;
ps2kbdTRIS      equ          TRISA         ; I/O do teclado
ps2kbdLAT       equ          LATA         ; Latch do teclado
ps2kbdPORT      equ          PORTA         ; Porta do teclaco
ps2kbdDATA      equ          2             ; bit data
ps2kbdCLOCK     equ          3             ; bit clock
ps2flags        res          1             ; flags do servico
ps2caps         equ          0             ; caps bit --> b'00000000'=OFF   b'00000001'=ON
ps2capsmsk      equ          1             ; mascara ---> b'00000001'
ps2ct           res          1             ; contador de bits
ps2cpy          res          1             ; 
ps2r1           res          1             ; 
ps2r2           res          1             ; 
ps2ledslat      res          1             ; latch dos LEDs do teclado
ps2buff         res          3             ; buffer de leitura
;
SR_1         RES          1             ; registrador auxiliar 1
SR_2         RES          1             ; registrador auxiliar 2
SR_3         RES          1             ; registrador auxiliar 3
SR_4         RES          1             ; registrador auxiliar 4
SR_5         RES          1             ; registrador auxiliar 5
LCD_COMMAND  RES          1             ; buffer de comando e dados
SR_TIPO      RES          1             ; tipo de LCD
SR_CLK       equ          3             ; em TRISx este bit deve ser saida
SR_DAT       equ          4             ; em TRISx este bit deve ser saida
SR_CHARS     equ          0             ; 0 - caracteres 5x7    1 - caracteres 5x10
#define      SR_TRIS      TRISB         ; I/O
#define      SR_DATA      LATB,SR_DAT   ; DATA
#define      SR_CLOCK     LATB,SR_CLK   ; CLOCK

               #include  "gpr.inc"       ; resgistradores de uso geral

pwron  CODE  0x0000
       goto  inicio
intrf  CODE  0x008
       retfie
; ----------------------------------------------------------------------
MAIN   CODE

           #include  "delay_services.inc"
           #include  "ps2keyboard_18f_services.inc"
           #include  "lcd_services.inc"
           #include  "hexag.asm"
           #include  "hex2ascii.inc"


msg001      SR_4MVC  ms001
ms001       DB  "TECLADO PS2 v1.1", 0

msg003      SR_4MVC   ms003
ms003       DB  "RC=0x", 0

msg004      SR_4MVC   ms004
ms004       DB  "ScanCd ", 0

msg007      SR_4MVC   ms007
ms007       DB  "Teclado OK!!!   ", 0

msg008      SR_4MVC   ms008
ms008       DB  "Restarting...   ", 0

msg999      SR_4MVC   ms999
ms999       DB  " Erro", 0


           LCD_LINES 0216

; ----------------------------------------------------------------------
inicio         movlw   0x0F                      ;  
               movwf   ADCON1                    ; 
               clrf    PORTA
               clrf    PORTB

               clrf    TRISA
               clrf    TRISB

               call    SR_4BITS                  ; configura o LCD
               call    SR_LCDRESET               ; apaga o LCD
               movlw   .1                        ; W=1
               movwf   RL                        ; indexador de linhas do LCD
               movwf   RC                        ; indexador de colunas do LCD

redo           SET_CURSOR 1, .1, "i"
               call    msg001                    ; msg="TECLADO PS2 v6.0"

               SET_CURSOR .2, .1, "i"            ; posiciona o cursor

               call    ps2kbd_init               ; inicializa o teclado
               call    ps2kbd_echo               ; pede um eco ao teclado
               movwf   R3                        ; salva resposta
               xorlw   0xEE                      ; verifica
               btfss   STATUS, Z                 ; teclado respondeu ao eco??? 
               goto    erro                      ; NAO --> nao ha comunicacao
               call    msg007                    ; SIM --> msg="Teclado OK!!!   "
               goto    segue                     ; continua o programa
erro           movf    R3, w                     ; pega a resposta
               call    shexa                     ; mostra no LCD
               call    msg999                    ; msg=" Erro"
loop           goto    loop                      ; nao posso continuar

segue          call    blink                     ; pisca os LEDs do teclado
 
getkbd         call    SR_LCDCLEAR               ; limpa o LCD

getkbd1        SET_CURSOR RL, RC, "r"            ; posiciona o cursor

getkbd2        call    ps2kbd_read               ; le o teclado
               movwf   kbdchar                   ; salva o caracter lido
               goto    funckeys                  ; verifica teclas de funcoes
showkbdchr     movf    kbdchar, w                ; recupera caracter lido
               call    SR_SENDW                  ; mostra no LCD
               incf    RC, f                     ; proxima coluna
               movlw   .250                      ; tempo
               call    delay_ms                  ; debounce para nao repetir
               goto    getkbd1                   ; vai ler o teclado

               ;
               ; Verifica tecla Delete
               ;
funckeys        movf   kbdchar, w
                onkey  _DEL, isDel
                goto   vCaps

isDel          call    SR_LCDCLEAR               ; SIM --> limpa o LCD
               movlw   .1
               movwf   RL
               movwf   RC
               goto    getkbd1                   ; fim

               ;
               ; Verifica tecla CAPS LOCK
               ;
vCaps          movlw   _CAPSLOCK
               xorwf   kbdchar, w
               btfss   STATUS, Z
               goto    vF1
               goto    getkbd1

               ;
               ; Verifica tecla F1
               ;
vF1            movf    kbdchar, w
               onkey  _F1, isF1
               goto    vF12
isF1           call    SR_LCDCLEAR
isF1a          SET_CURSOR .1, .1, "i"
               call    msg004                    ; msg="ScanCd "
               movlw   "0"
               call    SR_SENDW
               movlw   "x"
               call    SR_SENDW
               call    ps2kbd_getscan            ; le um Scan Code
               movwf   kbdchar                   ; salva
               movlw   .250
               call    delay_ms                  ; debounce
               movf    kbdchar, w
               xorlw   _F1                       ; Scan Code F1???
               bnz     isF1b                     ; NAO --> vai mostrar no LCD
               call    SR_LCDCLEAR               ; SIM --> limpa LCD
               goto    getkbd1                   ; fim               
isF1b          movf    kbdchar, w                ; pega Scan Code lido
               call    shexa                     ; mostra no LCD
               goto    isF1a                     ; 

               ;
               ; Verifica tecla F12
               ;
vF12           movf    kbdchar, w
               onkey  _F12, isF12
               goto    vBKSP 
isF12          call    blink   
               goto    getkbd1

               ;
               ; Verifica tecla  BACKSPACE
               ;
vBKSP          movf    kbdchar, w 
               onkey  _BKSP, isBKSP
               goto    vARROWR 
isBKSP         decfsz  RC, f
               goto    bkspclrc
               incf    RC, f
bkspclrc       SET_CURSOR RL, RC, "r"
               movlw   " "
               call    SR_SENDW
               SET_CURSOR RL, RC, "r"
               goto    getkbd1

               ;
               ; Verifica Seta a Direita
               ;
vARROWR        movf    kbdchar, w 
               onkey  _RIGHT, isARROWR 
               goto    vARROWL 
isARROWR       incf    RC, f
               movf    RC, w
               sublw   .16
               btfsc   STATUS, C                ; passou da coluna 16???
               goto    getkbd1                  ; NAO --> posiciona o cursor
_setCOL16      movlw   .16                      ; SIM --> mantem o cursor na col 16
               movwf   RC
               goto    getkbd1

               ;
               ; Verifica Seta a Esquerda
               ;
vARROWL        movf    kbdchar, w 
               onkey  _LEFT, isARROWL
               goto    vARROWU
isARROWL       decfsz  RC, f
               goto    getkbd1 
               incf    RC, f
               goto    getkbd1

               ;
               ; Verifica Seta para Cima
               ;
vARROWU        movf    kbdchar, w 
               onkey  _UP, isARROWU
               goto    vARROWD    
isARROWU       decfsz  RL, f
               goto    getkbd1 
               incf    RL, f
               goto    getkbd1 

               ;
               ; Verifica Seta para Baixo
               ;
vARROWD        movf    kbdchar, w 
               onkey  _DOWN, isARROWD
               goto    vEND    
isARROWD       incf    RL, f
               movf    RL, w
               sublw   .2
               btfsc   STATUS, C                ; passou da coluna 16???
               goto    getkbd1                  ; NAO --> posiciona o cursor
               movlw   .2                       ; SIM --> mantem o cursor na col 16
               movwf   RL
               goto    getkbd1                  ; posiciona o cursor

               ;
               ; Verifica tecla END
               ;
vEND           movf    kbdchar, w 
               onkey  _END,_setCOL16 

               ;
               ; Verifica tecla HOME
               ;
vHOME          movf    kbdchar, w 
               onkey  _HOME, gohome
               goto    vSYSR
gohome         movlw   .1
               movwf   RL
               movwf   RC 
               goto    getkbd1
    
               ;
               ; Verifica tecla SYSREQ
               ;
vSYSR          movf    kbdchar, w 
               onkey  _SYSREQ, reboot
               goto    vFunxxx
reboot         call    SR_LCDCLEAR
               call    msg008
               movlw   .1
               call    tempo
               goto    0x0000

vFunxxx

              goto    showkbdchr



; ----------------------------------------------------------------------
; Pisca os LEDs do teclado
; ----------------------------------------------------------------------
blink          movlw   .5
               movwf   R6
blink01        movlw   0x07
               call    ps2kbd_ledson     ; acende CAPS Lock         
               movlw   .50
               call    delay_ms
               movlw   0x07
               call    ps2kbd_ledsoff    ; apaga CAPS Lock         
               movlw   .50
               call    delay_ms 
               decfsz  R6, f
               goto    blink01
               return


; -------------------------------------------------------------------------------------
shexa          call    HEXAG
               movf    R1, w
               call    SR_SENDW
               movf    R0, w
               call    SR_SENDW
               return

; -------------------------------------------------------------------------------------
tempo          movwf   R6
               rlcf    R6, f
               rlcf    R6, f
tempo1         movlw   .250
               call    delay_ms
               decfsz  R6, f
               goto    tempo1
               return


           END
